/****************************************************************************
 *   Copyright (C) 2004 by Arboretum Systems, Inc.                          *
 *   Technical administrator: Alain Mazy, hav@mazy.be                       *
 *   Lead program designer: Georges Jaroslaw, gjaro@arboretum.com           *
 *                                                                          *
 *   HyperEngine-AV is a video, audio and text editor that can be used to   *
 *   record, import, edit, process and arrange together video, sound and    *
 *   text. These are all achieved in the program's free-form document       *
 *   window, with no limitations to the number of tracks.                   *
 *   The purpose of this program is to provide to users at all levels of    *
 *   expertise a readily accessible tool enabling them to create movies,    *
 *   slide shows or presentations.                                          *
 *                                                                          *
 *   HyperEngine-AV itself is free software. However, plug-ins that can be  *
 *   used as add-ons to HyperEngine-AV may or may not be free software, at  *
 *   the discretion of each plug-in developer and as specified by these     *
 *   plug-ins' respective licenses.                                         *
 *   You can redistribute HyperEngine-AV itself and/or modify it under the  *
 *   terms of the GNU General Public License as published by the Free       *
 *   Software Foundation; either version 2 of the License, or (at your      *
 *   option) any later version.                                             *
 *                                                                          *
 *   HyperEngine-AV is distributed in the hope that it will be useful, but  *
 *   WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for all applicable details.                 *
 *                                                                          *
 *   You should have received a copy of the GNU General Public License      *
 *   along with this program; if not, write to:                             *
 *   Free Software Foundation, Inc.,                                        *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.              *
 ****************************************************************************/

#ifndef MovieHelper_h
#include "MovieHelper.h"
#endif //MovieHelper_h

#ifndef TArmeDoc_h
#include "TArmeDoc.h"
#endif //TArmeDoc_h

#ifndef TVideoEffect_h 
#include "TVideoEffect.h"
#endif //TVideoEffect_h

#ifndef ZzHelpers_h
#include "ZzHelpers.h"
#endif //ZzHelpers_h

#ifndef Zz_PathHelpers_h
#include "Zz_PathHelpers.h"
#endif //Zz_PathHelpers_h

#ifndef ArmeDataFileRef_h
#include "ArmeDataFileRef.h"
#endif //ArmeDataFileRef_h

#ifndef MIProgressBarDlgManager_h
#include "MIProgressBarDlgManager.h"
#endif //MIProgressBarDlgManager_h

#ifndef ZzException_h
#include "ZzException.h"
#endif //ZzException_h

#undef VER_DEF_TOPIC
#define VER_DEF_TOPIC videoAlain


MovieHelper::MovieHelper()
{
	// get a list of the available effects
	OSErr myErr = QTNewAtomContainer(&fEffectsList);
	ZzAssert(myErr == 0);
		
	myErr = QTGetEffectsList(&fEffectsList, 0, 2, 0L);
	ZzAssert(myErr == 0);

}

MovieHelper::~MovieHelper()
{
	QTDisposeAtomContainer(fEffectsList);	
}

QTAtomContainer MovieHelper::GetEffectsList() const
{
	return fEffectsList;
}

MovieHelper theMovieHelper;



FSSpec Path2FSspec(const char* szFilePath, bool deleteIfFileExists)
{
	// some decls...
	FSRef 			folderRef;
	FSCatalogInfo	folderCatalogInfo;
	FSSpec			fileFSSpec;
	OSErr 			err;
	OSStatus		st;
	Str255			fileName;
	
	// first, we need to extract the path from the file name
	char	szFolder[kMaxPathLength];
	char	szFileName[kMaxFileLength];
	char 	szExtension[20];
	char 	filePath[kMaxPathLength];
	
	strcpy(filePath, szFilePath);
	ZzGetDirectory(szFolder, kMaxPathLength, szFilePath);
	ZzGetFileNameFromPath(szFileName, kMaxFileLength, szFilePath);
	ZzGetFileExtensionFromPath(szExtension, 20, szFilePath);
	// some API sometimes does not handle names longer than 30 char
	szFileName[20] = 0;
	ZzAppendExtension(szFileName, 30, szExtension);
	ZzAppendFileNameToPath(filePath, kMaxPathLength, szFolder, szFileName);
	
/*	int pathLen = strlen(szFilePath);
	int slashIndex = -1;
		
	for( i = pathLen-1; i >= 0; i-- )
	{
		if( szFilePath[i] == '/' )
		{
			slashIndex = i;
			break;
		}
	}
	
	// copy the folder name (including trailing slash) in 'folder'
	memcpy(szFolder,szFilePath,slashIndex+1);
	szFolder[slashIndex+1] = 0;
	
	// copy the fileName (excluding leading slash) in 'fileName'
	// first make sure the file name length fits in a pascal string
	assert(pathLen-(slashIndex+1) < 255);
	memcpy(szFileName,szFilePath+slashIndex+1,pathLen-(slashIndex+1));
	szFileName[pathLen-(slashIndex+1)] = 0;*/

	// do we need the trailing slash ?
	ZzString zzFolder(szFolder);
	
	st = ZzPathToFSRef(zzFolder.getCFString(),&folderRef);
//	st = FSPathMakeRef((const UInt8*)szFolder, &folderRef, NULL);
	ZzAssert(st==noErr);
	
	// we need to get the volume ref num for that FSRef
	err = FSGetCatalogInfo(&folderRef,kFSCatInfoVolume | kFSCatInfoNodeID,&folderCatalogInfo,NULL,NULL,NULL);
	ZzAssert(err==noErr);
	
	// create the file name in a pascal string
	CopyCStringToPascal(szFileName,fileName);
		
	// we need to create an FSSpec for the non-existing file in the folder...
	err = FSMakeFSSpec(folderCatalogInfo.volume,folderCatalogInfo.nodeID,fileName,&fileFSSpec);
	if (err == -37)
		vernl(0, "Fatal error in Path2FSspec : " << fileName << " : bad filename (too long ?)\n");

	ZzAssert(err==-43 || err==0);

	if ((err == noErr) && (deleteIfFileExists))
		{
		err = FSpDelete(&fileFSSpec);
		if (err != noErr)
			vernl(4, "Error in Path2FSspec : " << fileName << " : can't delete the file\n");
		}

    return fileFSSpec;
}

// THIS FUNCTION IS VERY VERY HACKY
/*
Note : Before using the returned value, the calling function should always ensure that err==noErr
*/
FSSpec FSSpecFromNonExistingFilePath(char* szFilePath, OSType creator, OSType type, OSErr& err)
{
	// some decls...
	FSRef 			folderRef;
	FSCatalogInfo	folderCatalogInfo;
	FSSpec			fileFSSpec;
	OSStatus		st;
	Str255			fileName;
	
	// first, we need to extract the path from the file name
	char	szFolder[kMaxPathLength];
	char	szFileName[kMaxFileLength];
	char 	szExtension[20];
	char 	filePath[kMaxPathLength];
	
	strcpy(filePath, szFilePath);
	ZzGetDirectory(szFolder, kMaxPathLength, szFilePath);
	ZzGetFileNameFromPath(szFileName, kMaxFileLength, szFilePath);
	ZzGetFileExtensionFromPath(szExtension, 20, szFilePath);
	// some API sometimes does not handle names longer than 30 char
	szFileName[20] = 0;
	ZzAppendExtension(szFileName, 30, szExtension);
	ZzAppendFileNameToPath(filePath, kMaxPathLength, szFolder, szFileName);
	
/*	int pathLen = strlen(szFilePath);
	int slashIndex = -1;
		
	for( i = pathLen-1; i >= 0; i-- )
	{
		if( szFilePath[i] == '/' )
		{
			slashIndex = i;
			break;
		}
	}
	
	// copy the folder name (including trailing slash) in 'folder'
	memcpy(szFolder,szFilePath,slashIndex+1);
	szFolder[slashIndex+1] = 0;
	
	// copy the fileName (excluding leading slash) in 'fileName'
	// first make sure the file name length fits in a pascal string
	assert(pathLen-(slashIndex+1) < 255);
	memcpy(szFileName,szFilePath+slashIndex+1,pathLen-(slashIndex+1));
	szFileName[pathLen-(slashIndex+1)] = 0;*/

	// do we need the trailing slash ?
	ZzString zzFolder(szFolder);
	
	st = ZzPathToFSRef(zzFolder.getCFString(),&folderRef);
//	st = FSPathMakeRef((const UInt8*)szFolder, &folderRef, NULL);
	ZzAssert(st==noErr);
	
	// we need to get the volume ref num for that FSRef
	err = FSGetCatalogInfo(&folderRef,kFSCatInfoVolume | kFSCatInfoNodeID,&folderCatalogInfo,NULL,NULL,NULL);
	if (ThrowOnOSStatus(err))
		goto exit_on_error;
	
	// create the file name in a pascal string
	CopyCStringToPascal(szFileName,fileName);
		
	// we need to create an FSSpec for the non-existing file in the folder...
	err = FSMakeFSSpec(folderCatalogInfo.volume,folderCatalogInfo.nodeID,fileName,&fileFSSpec);
	if ((err != -43) && (err != 0))
	{
		if (ThrowOnOSStatus(err))
			goto exit_on_error;
	}

	err = FSpCreate(&fileFSSpec,creator,type,smSystemScript);
	if(err == dupFNErr)
	{
		vernl(4, "Create File : " << fileName << " : file already exists. It will be deleted.\n");
		// GP : use FSPDelete instead of remove(filePath);
		err = FSpDelete(&fileFSSpec);
		if (ThrowOnOSStatus(err))
			goto exit_on_error;
		else
		{
			// GP : create the new file.
			err = FSpCreate(&fileFSSpec,creator,type,smSystemScript);
			if (ThrowOnOSStatus(err))
				goto exit_on_error;
		}
	}
	else
	{
		if (ThrowOnOSStatus(err))
			goto exit_on_error;
	}
	
exit_on_error :
    return fileFSSpec;
}

void SaveMovie(const Movie movie, char* szFilePath)
{
	OSErr err = noErr;
    FSSpec fsFSSpec = FSSpecFromNonExistingFilePath(szFilePath, FOUR_CHAR_CODE('TVOD'), FOUR_CHAR_CODE('MooV'), err);
    if (ThrowOnOSStatus(err))
		return;
	
    FlattenMovie(movie, 0, &fsFSSpec, FOUR_CHAR_CODE('TVOD'), smCurrentScript, createMovieFileDeleteCurFile, 0, 0);
}


Movie LoadMovie(const char* szFilePath, short* movieResFile, short* movieResId, ArmeErr* err, FSSpec* fsSpec, bool forEditing)
{
    Movie retMovie;
    OSErr res1, res2, res3, res4;
    Str255 theMovieName;
    Boolean thefWasChanged;
    FSRef fsRef;
    FSSpec aFsSpec;

	if (fsSpec == NULL)
	{
		fsSpec = &aFsSpec;
	}

	res1 = FSPathMakeRef ((unsigned char*)szFilePath, &fsRef, FALSE);
    if (ThrowOnOSStatus(res1))
	{
		err->type = kArmeErrTVideoClipCannotOpenVideoFile;
		vernl(0, "Fatal error in Load Movie : " << szFilePath << " : bad filename\n");
        return NULL;
	}
	
    res2 = FSGetCatalogInfo (&fsRef, kFSCatInfoNone, NULL, NULL, fsSpec, NULL);
    if (ThrowOnOSStatus(res2)) 
	{
		err->type = kArmeErrTVideoClipCannotOpenVideoFile;
		vernl(0, "Fatal error in Load Movie : " << szFilePath << " : bad filename\n");
        return NULL;
	}

	SInt8 permission = forEditing ? fsRdWrPerm : fsRdPerm;
    res3 = OpenMovieFile ( fsSpec, movieResFile, permission);
    if (ThrowOnOSStatus(res3)) 
	{
		err->type = kArmeErrTVideoClipCannotOpenVideoFile;
		vernl(0, "Fatal error in Load Movie : " << szFilePath << " : could not open file\n");
        return NULL;
	}
        			
	verntl(dataFileRef, 5, "Opening WorkingMaster.mov");
    res4 = NewMovieFromFile( &retMovie, *movieResFile, movieResId, theMovieName, newMovieActive, &thefWasChanged);
	verntl(dataFileRef, 5, "Closing WorkingMaster.mov");
    CloseMovieFile(*movieResFile); 
	
    if (res4 != noErr) 
    {	
		vernl(0, "Warning in Load Movie : " << szFilePath << " : resource not found (" << res4 << ") retrying\n");
		//Let's try again now that the file has been closed properly
		res4 = NewMovieFromFile( &retMovie, *movieResFile, movieResId, theMovieName, newMovieActive, &thefWasChanged);
		if (ThrowOnOSStatus(res4))
		{
			err->type = kArmeErrTVideoClipCannotOpenVideoFile;
			vernl(0, "Fatal error in Load Movie : " << szFilePath << " : resource not found (" << res4 << ")\n");
			return NULL;
		}
		CloseMovieFile(*movieResFile); 
    }

    return retMovie;
}

Movie LoadMovie(const FSSpec& movieFile, int trackIndex, short* movieResFile, ArmeErr* err, bool forEditing)
{
    Movie retMovie;
    OSErr res;
    short movieResID = 0;
    Str255 theMovieName;
    Boolean thefWasChanged;

	SInt8 permission = forEditing ? fsRdWrPerm : fsRdPerm;
    res = OpenMovieFile ( &movieFile, movieResFile, permission);
    if (ThrowOnOSStatus(res)) 
	{
		err->type = kArmeErrTVideoClipCannotOpenVideoFile;
		vernl(0, "Fatal error in Load Movie : " << movieFile.name << " : could not open file\n");
        return NULL;
	}
        			
    res = NewMovieFromFile( &retMovie, *movieResFile, &movieResID, theMovieName, newMovieActive, &thefWasChanged);
    CloseMovieFile(*movieResFile);
    if (ThrowOnOSStatus(res)) 
    {	
		err->type = kArmeErrTVideoClipCannotOpenVideoFile;
		vernl(0, "Fatal error in Load Movie : " << movieFile.name << " : resource not found (" << res << ")\n");
        return NULL;
    }

    /* if we want to return a movie with just trackIndex in it...
    if (retMovie and (trackIndex > 0))
    {
        long i, j, trackCount = ::GetMovieTrackCount(retMovie);
        // delete all above
        for (i = trackCount; i > trackIndex; i--)
            ::DisposeMovieTrack(::GetMovieIndTrack(retMovie, i));
        // delete all below
        for (j = trackIndex - 1; j > 0; j--)
            ::DisposeMovieTrack(::GetMovieIndTrack(retMovie, j));
    }
    */
	ZzAssert(retMovie);
	   
    return retMovie;
}

OSErr CopyMovieSegment(Movie destMovie, const Movie srcMovie, TimeValue destStartTime, TimeValue srcStartTime, TimeValue srcDuration)
{
    OSErr res;

    //we first insert the selection in the dest Movie and then delete what was at this place before
    //by this way, source and destination can have a common part in the same movie
    res = InsertMovieSegment(srcMovie, destMovie, srcStartTime, srcDuration, destStartTime);
	if(ThrowOnOSStatus(res))
		return res;
	
	// express duration in destination time scale
	TimeValue destDuration = long(SInt64(srcDuration) * SInt64(GetMovieTimeScale(destMovie)) / SInt64(GetMovieTimeScale(srcMovie)));

    res = DeleteMovieSegment(destMovie, destStartTime + destDuration, destDuration);
	ThrowOnOSStatus(res);
    return res;
}

OSErr CopyTrackSegment(Movie destMovie, Track destTrack, const Movie srcMovie, const Track srcTrack, TimeValue destStartTime, TimeValue srcStartTime, TimeValue srcDuration)
{
    OSErr res;

	verntl(videoAlain, 5, "CopyTrackSegment() from ( " << srcStartTime << ", " << srcDuration << " to " << destStartTime << ")");


    //we first insert the selection in the dest Track and then delete what was at this place before
    //by this way, source and destination can have a common part in the same track
    res = InsertTrackSegment(srcTrack, destTrack, srcStartTime, srcDuration, destStartTime);
	ThrowOnOSStatus(res);	
	
	// express duration in destination time scale
	TimeValue destDuration = long(double(srcDuration) * double(GetMovieTimeScale(destMovie)) / double(GetMovieTimeScale(srcMovie)));
	
    res = DeleteTrackSegment(destTrack, destStartTime + destDuration, destDuration);
	ThrowOnOSStatus(res);

    return res;
}

Track GetCopyOfTrack(Movie movie, const Track srcTrack, int trackWidth, int trackHeight)
{
	Track newTrack = CreateNewTrackInMovie(movie, GetMovieTimeScale(movie), trackWidth, trackHeight);
	CopyTrackSegment(movie, newTrack, movie, srcTrack, 0, 0, GetTrackDuration(srcTrack));
	
	return newTrack;
}

ArmeDataFileRef* RenderMovieIntoSingleTrackMovie(ArmeErr &armeErr, Movie movie, TArmeDoc* inDoc, MIProgressBarDlgManager* prog, ArmeTime startTime, ArmeTime endTime, bool reverse, bool useCustomAlgo, bool highRes, bool stillClipToScale)
{
    OSStatus err = noErr;
	OSErr sysErr = noErr;
	Movie srcMovie = movie;
	ArmeDataFileRef* fileRef = NULL;
	
	if (reverse || useCustomAlgo)
	{
	
		bool bProgressDlgWasCreatedInThisFunction = false;
		
		if (!prog)
		{
			ZzAssert(0);
		//	prog = new MIProgressBarDlgManager("Rendering");
		//	bProgressDlgWasCreatedInThisFunction = true;
		//	prog->DisplayDlg("RenderMovieIntoSingleTrackMovie : you should implement a dlg in a higher level function", false);
		}
	
		// rendering settings which should be in project settings at some point
		long projectFPS = long(inDoc->fMasterMovie->GetFPS()); 
		TimeScale projectTimeScale = GetMovieTimeScale(inDoc->fMasterMovie->GetQTMovie());
		OSType compressor = kMotionJPEGACodecType;  //this algo always use MJPEG  //$$TODO : try DV
		Rect projectRect = {0, 0, inDoc->fMasterMovie->GetWorkingHeight(), inDoc->fMasterMovie->GetWorkingWidth()};
//		::GetMovieBox(srcMovie, &projectRect);
		
		fileRef = inDoc->GetNewVideoDataFileRef();
		char path[kMaxPathLength + 1] = { 0 };
		fileRef->GetFilePath(path);
		FSSpec newSpec = FSSpecFromNonExistingFilePath(path, FOUR_CHAR_CODE('TVOD'), FOUR_CHAR_CODE('MooV'), sysErr);
			
		// grab srcMovie's current port so we don't leave it dangling
		CGrafPtr srcPort;
		GDHandle srcDevice;
		::GetMovieGWorld(srcMovie, &srcPort, &srcDevice);
	
		// create a buffer for source to play into
		GWorldPtr sourceWorld = nil;
		
		err = ::NewGWorld(&sourceWorld, 32, &projectRect, nil, nil, 0);
		if ( err ) {
			vernl(0, "Render Movie Into Single Track : Could not create GWorld");
			::FSpDelete(&newSpec);
			return NULL;
		}
		
		::SetMovieGWorld(srcMovie, sourceWorld, ::GetGWorldDevice(sourceWorld));
		PixMapHandle sourcePixMap = ::GetPortPixMap(sourceWorld);
		::SetMovieTimeValue(srcMovie, 0);
		err = ::UpdateMovie(srcMovie);
		if ( err ) {
			vernl(0, "Render Movie Into Single Track : Could not update Movie");
			::FSpDelete(&newSpec);
			::SetMovieGWorld(srcMovie, srcPort, srcDevice);
			return NULL;
		}
		::MoviesTask(srcMovie, 0);

		::SetMovieBox(srcMovie, &projectRect);
/*		Rect tempRect;
		::GetMovieBox(srcMovie, &tempRect);*/

		
		// create a compressor using that buffer
		CodecQ spatialQuality = codecNormalQuality;
		CodecQ temporalQuality = 0; // NO temporal compression!
		long maxCompressedFrameSize;
		err = ::GetMaxCompressionSize(sourcePixMap, &projectRect, 0, spatialQuality, compressor,
				bestSpeedCodec, &maxCompressedFrameSize);
		if ( err ) {
			vernl(0, "Render Movie Into Single Track : Could not get max compression size");
			::FSpDelete(&newSpec);
			::SetMovieGWorld(srcMovie, srcPort, srcDevice);
			return NULL;
		}
		
		Handle compressedDataH = ::NewHandle(maxCompressedFrameSize);
		if ( !compressedDataH ) {
			vernl(0, "Render Movie Into Single Track : Could not create a new handle");
			::FSpDelete(&newSpec);
			::SetMovieGWorld(srcMovie, srcPort, srcDevice);
			return NULL;
		}
		
		Handle compressedIDH = ::NewHandle(sizeof(ImageDescription));
		if ( !compressedIDH ) {
			vernl(0, "Render Movie Into Single Track : Could not create a new handle");
			::DisposeHandle(compressedDataH);
			::FSpDelete(&newSpec);
			::SetMovieGWorld(srcMovie, srcPort, srcDevice);
			return NULL;
		}
		
		ImageSequence sequenceID;
		err = ::CompressSequenceBegin(&sequenceID, sourcePixMap, nil, &projectRect, nil, 0, 
				compressor, bestSpeedCodec, spatialQuality, temporalQuality, projectFPS, nil,
				codecFlagUpdatePrevious | codecFlagWasCompressed, (ImageDescriptionHandle)compressedIDH);
		if (err)
			{
			vernl(0, "Render Movie Into Single Track : Could not start sequence compression");
			::DisposeHandle(compressedIDH);
			::DisposeHandle(compressedDataH);
			::FSpDelete(&newSpec);
			::SetMovieGWorld(srcMovie, srcPort, srcDevice);
			return nil;
			}
	
		// create new movie to return
		Movie newMovie = nil; 
		short newRefNum;
		err = ::CreateMovieFile(&newSpec,'TVOD', 0, 
				createMovieFileDeleteCurFile | createMovieFileDontCreateResFile,
				&newRefNum, &newMovie);
                SetMovieTimeScale(newMovie, inDoc->fMasterMovie->GetVideoTimeScale());
		if (err)
			{
			vernl(0, "Render Movie Into Single Track : Could not create movie file");
			::CDSequenceEnd(sequenceID);
			::DisposeHandle(compressedIDH);
			::DisposeHandle(compressedDataH);
			::FSpDelete(&newSpec);
			::SetMovieGWorld(srcMovie, srcPort, srcDevice);
			return nil;
			}
		Track newTrack = ::NewMovieTrack(newMovie,
					(long)(projectRect.right - projectRect.left) << 16,
					(long)(projectRect.bottom - projectRect.top) << 16, 0);
		Media newMedia = ::NewTrackMedia(newTrack, VIDEO_TYPE, projectTimeScale, 0, 0);
		OSErr err = ::BeginMediaEdits(newMedia);
		if (ThrowOnOSStatus(err)) {
			vernl(0, "Render Movie Into Single Track : Could not begin media edits");
			::CloseMovieFile(newRefNum);
			::DisposeMovie(newMovie);
			::CDSequenceEnd(sequenceID);
			::DisposeHandle(compressedIDH);
			::DisposeHandle(compressedDataH);
			::FSpDelete(&newSpec);
			::SetMovieGWorld(srcMovie, srcPort, srcDevice);
			return nil;
		}
		
		
		// render srcMovie in newMovie
		TimeValue currentTime = 0;
		TimeValue finishTime = ::GetMovieDuration(srcMovie);
		if (startTime >= ARME_TIME_ZERO)
			currentTime = TimeValueFromArmeTime2(srcMovie, inDoc, startTime);
		if (endTime > ARME_TIME_ZERO)
			finishTime = TimeValueFromArmeTime2(srcMovie, inDoc, endTime);
		TimeValue sourceDuration = ::GetMovieTimeScale(srcMovie) / projectFPS;
		TimeValue newDuration = projectTimeScale / projectFPS;
		TimeValue tvStartTime = currentTime;
	
		do
			{
			// draw new frame
			if (reverse)
				::SetMovieTimeValue(srcMovie, finishTime - (currentTime - tvStartTime) - sourceDuration);
			else
				::SetMovieTimeValue(srcMovie, currentTime);
			err = ::UpdateMovie(srcMovie);
			if (ThrowOnOSStatus(err)) {
				vernl(0, "Render Movie Into Single Track : Could not update movie");
				::EndMediaEdits(newMedia);
				::CloseMovieFile(newRefNum);
				::DisposeMovie(newMovie);
				::CDSequenceEnd(sequenceID);
				::DisposeHandle(compressedIDH);
				::DisposeHandle(compressedDataH);
				::FSpDelete(&newSpec);
				::SetMovieGWorld(srcMovie, srcPort, srcDevice);
				return nil;
			}
			
			::MoviesTask(srcMovie, 0);
		
			// compress it
			short syncFlag;
			long dataSize;
			UInt8 similarity;
			err = ::CompressSequenceFrame(sequenceID, sourcePixMap, &projectRect,
					codecFlagUpdatePrevious + codecFlagWasCompressed, *compressedDataH, &dataSize, &similarity, nil);
			if (ThrowOnOSStatus(err)) {
				vernl(0, "Render Movie Into Single Track : Could not compress sequence frame");
				::EndMediaEdits(newMedia);
				::CloseMovieFile(newRefNum);
				::DisposeMovie(newMovie);
				::CDSequenceEnd(sequenceID);
				::DisposeHandle(compressedIDH);
				::DisposeHandle(compressedDataH);
				::FSpDelete(&newSpec);
				::SetMovieGWorld(srcMovie, srcPort, srcDevice);
				return nil;
			}
			
			
			syncFlag = similarity ? mediaSampleNotSync : 0;
	
			// add to new movie
			err = ::AddMediaSample(newMedia, compressedDataH, 0, dataSize, newDuration,
					(SampleDescriptionHandle)compressedIDH, 1, syncFlag, nil);
			if (ThrowOnOSStatus(err)) {
				vernl(0, "Render Movie Into Single Track : Could not add media sample");
				::EndMediaEdits(newMedia);
				::CloseMovieFile(newRefNum);
				::DisposeMovie(newMovie);
				::CDSequenceEnd(sequenceID);
				::DisposeHandle(compressedIDH);
				::DisposeHandle(compressedDataH);
				::FSpDelete(&newSpec);
				::SetMovieGWorld(srcMovie, srcPort, srcDevice);
				return nil;
			}
			
		
			// skip to next frame
			currentTime += sourceDuration;
	
				prog->Update(float(currentTime - tvStartTime)/float(finishTime - tvStartTime));
			}
		while (!err and (currentTime < finishTime)); // the rendering is always longer than the source; so let's try this
	//    while (!err and (currentTime <= finishTime));
	
		// all done
		err = ::EndMediaEdits(newMedia);
		if (ThrowOnOSStatus(err)) {
			vernl(0, "Render Movie Into Single Track : Could not end media edits");
			::CloseMovieFile(newRefNum);
			::DisposeMovie(newMovie);
			::CDSequenceEnd(sequenceID);
			::DisposeHandle(compressedIDH);
			::DisposeHandle(compressedDataH);
			::FSpDelete(&newSpec);
			::SetMovieGWorld(srcMovie, srcPort, srcDevice);
			return nil;
		}
		
		err = ::CDSequenceEnd(sequenceID);
		if (ThrowOnOSStatus(err)) {
			vernl(0, "Render Movie Into Single Track : Could not end sequence");
			::CloseMovieFile(newRefNum);
			::DisposeMovie(newMovie);
			::DisposeHandle(compressedIDH);
			::DisposeHandle(compressedDataH);
			::FSpDelete(&newSpec);
			::SetMovieGWorld(srcMovie, srcPort, srcDevice);
			return nil;
		}
		
		
		err = ::InsertMediaIntoTrack(newTrack, 0, 0, ::GetMediaDuration(newMedia), fixed1);
		if (ThrowOnOSStatus(err)) {
			vernl(0, "Render Movie Into Single Track : Could not insert media into track");
			::CloseMovieFile(newRefNum);
			::DisposeMovie(newMovie);
			::DisposeHandle(compressedIDH);
			::DisposeHandle(compressedDataH);
			::FSpDelete(&newSpec);
			::SetMovieGWorld(srcMovie, srcPort, srcDevice);
			return nil;
		}
		
		short resID = movieInDataForkResID;
		err = ::AddMovieResource(newMovie, newRefNum, &resID, "\prendered");
		if (ThrowOnOSStatus(err)) {
			vernl(0, "Render Movie Into Single Track : Could not add movie resource");
			::CloseMovieFile(newRefNum);
			::DisposeMovie(newMovie);
			::DisposeHandle(compressedIDH);
			::DisposeHandle(compressedDataH);
			::FSpDelete(&newSpec);
			::SetMovieGWorld(srcMovie, srcPort, srcDevice);
			return nil;
		}
		
		::DisposeHandle(compressedDataH);
		::DisposeHandle(compressedIDH);
		::SetMovieGWorld(srcMovie, srcPort, srcDevice);
		::DisposeGWorld(sourceWorld);
	
		// return the updated movie in ioMovie
	/*	::DisposeMovie(srcMovie);
		ioMovie = newMovie;*/
		::DisposeMovie(newMovie);
		
		if (bProgressDlgWasCreatedInThisFunction)
		{
			prog->Delete();
			delete prog;
		}
	}
	else
	{
		//new code AM to take into account the working codec
		try
		{
			FSSpec dstFile;
		//	ZzAppendFileNameToPath(, kMaxPathLength, doc->GetProjectFolderPath(), clipName);
			char sourceFile[kMaxPathLength];
			fileRef = inDoc->GetNewVideoDataFileRef();
			fileRef->GetFilePath(sourceFile);
			dstFile = FSSpecFromNonExistingFilePath(sourceFile, FOUR_CHAR_CODE('TVOD'), FOUR_CHAR_CODE('MooV'), sysErr);
			if (ThrowOnOSStatus(sysErr))
				goto exit_on_error;
	
			TimeValue start = inDoc->TimeValueFromArmeTime(startTime);
			TimeValue end = inDoc->TimeValueFromArmeTime(endTime);
			
			// the movie needs to be at least one frame long
			if (start == end)
			{
				end += inDoc->GetVideoFrameDuration();
			}
		
			ZzAssert(start != end);	
			Movie copy = NewMovie(newMovieActive);
			SetMovieTimeScale(copy, inDoc->fMasterMovie->GetVideoTimeScale());  
			OSErr res = CopyMovieSegment(copy, movie, 0, start, end - start);
			ThrowOnOSStatus(res);
	
			SetMovieProgressProc(copy, QTMovieProgressProc, (long)prog);
			if (stillClipToScale)  //if we are rendering a still clip, use highest quality codec
				res = ConvertMovieToFile(copy, 0, &dstFile, 0, 0, -1, 0, 0, GetCodecInstance(inDoc->fMasterMovie->GetWorkingCodecTypeForPictures())); 
			else if (highRes)
				res = ConvertMovieToFile(copy, 0, &dstFile, 0, 0, -1, 0, 0, GetCodecInstance(inDoc->fMasterMovie->GetWorkingCodecType())); 
			else
				res = ConvertMovieToFile(copy, 0, &dstFile, 0, 0, -1, 0, 0, GetCodecInstance(inDoc->fMasterMovie->GetPreviewCodecType())); 
			DisposeMovie(copy);
	
			ThrowOnOSStatus(res);
		}
		catch (ZzException* e)
		{
			if (fileRef)
			{
				inDoc->RemoveVideoDataFileRef(fileRef);
				delete fileRef;
			}
			
			throw;
		}
	}

exit_on_error :
    return fileRef;
}

Track CreateNewTrackInMovie(Movie destMovie, TimeScale scale, int trackWidth, int trackHeight, OSType trackType)
{
	Track newTrack = NewMovieTrack(destMovie, trackWidth << 16, trackHeight << 16, kNoVolume);
	OSErr res = GetMoviesError();
    ThrowOnOSStatus(res);
	
	NewTrackMedia(newTrack, trackType, scale, 0, 0);
	res = GetMoviesError();
    ThrowOnOSStatus(res);
	return newTrack;
}

Track InsertMovieSegmentAsNewTrackInMovie(Movie destMovie, const Movie srcMovie, TimeValue destStartTime, TimeValue srcStartTime, TimeValue srcDuration, int trackWidth, int trackHeight)
{
	OSErr res;
	Track srcTrack = GetFirstVideoTrack(srcMovie);
	OSType trackType;
	GetMediaHandlerDescription(GetTrackMedia(srcTrack), &trackType, 0, 0);

	Track newTrack = CreateNewTrackInMovie(destMovie, GetMovieTimeScale(destMovie), trackWidth, trackHeight, trackType);
	res = CopyTrackSegment(destMovie, newTrack, srcMovie, srcTrack, destStartTime, srcStartTime, srcDuration);
	ThrowOnOSStatus(res);	

	return newTrack;
}

Track InsertTrackSegmentAsNewTrackInMovie(Movie destMovie, const Movie srcMovie, int trackIndex, TimeValue destStartTime, TimeValue srcStartTime, TimeValue duration, int trackWidth, int trackHeight)
{
	OSErr res;
	Track srcTrack = ::GetMovieIndTrack(srcMovie, trackIndex);
	ZzAssert(srcTrack);
	OSType trackType;
	::GetMediaHandlerDescription(::GetTrackMedia(srcTrack), &trackType, 0, 0);
	int srcLayer = ::GetTrackLayer(srcTrack);

	Track newTrack = CreateNewTrackInMovie(destMovie, GetMovieTimeScale(destMovie), trackWidth, trackHeight, trackType);
	::SetTrackLayer(newTrack, srcLayer);

	res = CopyTrackSegment(destMovie, newTrack, srcMovie, srcTrack, destStartTime, srcStartTime, duration);
	ThrowOnOSStatus(res);	

	return newTrack;
}

Track InsertTrackSegmentAsNewTrackInMovie(Movie destMovie, const Track srcTrack, TimeValue destStartTime, TimeValue srcStartTime, TimeValue duration, int trackWidth, int trackHeight)
{
	OSErr res;
	OSType trackType;
	::GetMediaHandlerDescription(::GetTrackMedia(srcTrack), &trackType, nil, nil);
	int srcLayer = ::GetTrackLayer(srcTrack);

	Track newTrack = CreateNewTrackInMovie(destMovie, GetMovieTimeScale(destMovie), trackWidth, trackHeight, trackType);
	::SetTrackLayer(newTrack, srcLayer);

	res = CopyTrackSegment(destMovie, newTrack, destMovie, srcTrack, destStartTime, srcStartTime, duration);
	ThrowOnOSStatus(res);
	return newTrack;
}


Movie CreatePreviewMovie(const Movie movie, char* szFilePath, int width, int height)
{
	Movie preview = NewMovie(newMovieActive);
        SetMovieTimeScale(movie, GetMovieTimeScale(movie));
	InsertMovieSegmentAsNewTrackInMovie(preview, movie, 0, 0, GetMovieDuration(movie), width, height);
	SaveMovie(preview, szFilePath);
	
	return preview;
}


Track GetFirstVideoTrack(const Movie movie)
{
	int iTrack;

	for (iTrack = 1; iTrack <= GetMovieTrackCount(movie); iTrack++) 
	{
		OSType trackType;
		Track track = GetMovieIndTrack(movie, iTrack);

		GetMediaHandlerDescription(GetTrackMedia(track), &trackType, 0, 0);
                
                switch (trackType)
                {
                    case VideoMediaType:
                    case FlashMediaType:
                    case ThreeDeeMediaType:
                    case TimeCodeMediaType:
                    case SpriteMediaType:
                        return track;
                    case TextMediaType:
                    case MPEGMediaType:

                    case SoundMediaType:
                    case MusicMediaType:
                    case MovieMediaType:
                    case TweenMediaType:
                    default:
                        break;
               }
	}
	
	return NULL;
}

void PrerollMovieArmeTime(const Movie movie, const TArmeDoc* doc, const ArmeTime time)
{
	return;

	TimeRecord movieTime;
	TimeValue currentTime = doc->TimeValueFromArmeTime(time);
	GetMovieTime(movie, &movieTime);

//	fprintf(fp, "time : %d currentTime : %d movieTime : %d\n", int(time.data), int(currentTime), int(movieTime.value.lo));
//	fclose(fp);
	
	assert(movieTime.value.hi == 0); // "setMovieArmeTime is not working with big time values !"); 
	movieTime.value.lo = currentTime;
	
	SetMovieTime(movie, &movieTime);
	if (!doc->IsPlaying())
	{
		if (doc->fViewerIsVisible)
		{
			SetMoviePosterTime(movie, currentTime);
			ShowMoviePoster(movie);
		}
		PrePrerollMovie(movie, currentTime, 1 << 16, NULL, NULL);
		PrerollMovie(movie, currentTime, 1 << 16);
		/* $$$marcp : ShowMoviePoster() makes an unwanted white zone on the upper left corner of the screen when creating a new document.
		 ShowMoviePoster(movie);
		res = GetMoviesError();*/
	}
}
	
void SetMovieArmeTime(const Movie movie, const TArmeDoc* doc, const ArmeTime time)
{
	TimeRecord movieTime;
	TimeValue currentTime = doc->TimeValueFromArmeTime(time);
	GetMovieTime(movie, &movieTime);

	assert(movieTime.value.hi == 0); // "setMovieArmeTime is not working with big time values !"); 
	movieTime.value.lo = currentTime;
	
	
	SetMovieTime(movie, &movieTime);

	if (!doc->IsPlaying())
	{
		if (doc->fViewerIsVisible)
		{
			SetMoviePosterTime(movie, currentTime);
			ShowMoviePoster(movie);
		}
	}
	/*	PrePrerollMovie(movie, currentTime, 1 << 16, NULL, NULL);
		PrerollMovie(movie, currentTime, 1 << 16);
		/// $$$marcp : ShowMoviePoster() makes an unwanted white zone on the upper left corner of the screen when creating a new document.
		// ShowMoviePoster(movie);
		//res = GetMoviesError();
	} */
}

ArmeTime GetMovieArmeTime(const Movie movie, const TArmeDoc* doc)
{
	return doc->SecondsToArmeTime(GetMovieTimeSec(movie));
}

float GetMovieTimeSec(const Movie movie)
{
	TimeRecord movieTime;
	GetMovieTime(movie, &movieTime);

	assert(movieTime.value.hi == 0); // "getMovieTimeSec is not working with big time values !"); 
	return float(movieTime.value.lo)/float(movieTime.scale);
}

/*
Correct the current time of the movie to make sure that it plays at the same rate as the sound does.
Actually, we modify the current time only if the offset between video and sound is bigger than 1/50th of
a second (below that, you can't see it).
*/
void AdjustMovieArmeTime(Movie movie, const TArmeDoc* doc, const ArmeTime time)
{
	float timeSec = doc->ArmeTimeToSeconds(time);
	float offset = GetMovieTimeSec(movie) - timeSec;
	float frameDurationSec = float(doc->GetVideoFrameDuration()) / float(doc->GetVideoTimeScale());
	if ((offset > frameDurationSec) || (offset < -frameDurationSec))
		SetMovieArmeTime(movie, doc, time);
}

Track CreateEffectOnMovie(Movie movieSource, Track trackA, Track trackB)
{
	Track trackSource[2];
	Media mediaSource;
	int nbTrackSource;
	TimeScale timeScaleSource;
	Fixed widthSource, heightSource;
	OSErr  res = noErr;
	ImageDescriptionHandle		mySampleDesc = NULL;

	trackSource[0] = trackA;
	if (trackB)
	{
		trackSource[1] = trackB;
		nbTrackSource = 2;
	}
	else
		nbTrackSource = 1;

	// prise info sur le track video source
	mediaSource = GetTrackMedia(trackSource[0]);
	timeScaleSource = GetMediaTimeScale(mediaSource);
	GetTrackDimensions(trackSource[0], &widthSource, &heightSource);

	/*************************************************************************/
	/* creation de l'effet                                                   */
	/*************************************************************************/

	// creation de la piste d'effet
	Track trackEffect;
	Media mediaEffect;

    trackEffect = NewMovieTrack(movieSource, widthSource, heightSource, kNoVolume);

	Handle    dataRef = nil;
	Handle    hMovieData = NewHandle(0);

	// Construct the Handle data reference
	res = PtrToHand( &hMovieData, &dataRef, sizeof(Handle));
    ThrowOnOSStatus(res);

	mediaEffect = NewTrackMedia(trackEffect, VideoMediaType, timeScaleSource, dataRef, HandleDataHandlerSubType); 

	// creation d'un effectDescription
	QTAtomContainer		myEffectDesc = NULL, myInputMap = NULL;
	OSType				myType;

	res = QTNewAtomContainer(&myEffectDesc);
    ThrowOnOSStatus(res);

	bool bDialog = true;
	if (bDialog)
	{
		QTAtomContainer effectList;
		QTParameterDialog createdDialog;
		OSErr result=noErr;

		res = QTGetEffectsList(&effectList, 0, nbTrackSource, 0);  
		ThrowOnOSStatus(res);
		res = QTCreateStandardParameterDialog (effectList, myEffectDesc, pdOptionsAllowOptionalInterpolations, &createdDialog); 
		ThrowOnOSStatus(res);

//		res = QTCreateStandardParameterDialog (effectList, myEffectDesc, pdOptionsAllowOptionalInterpolations, createdDialog); 		
		while (result == noErr)
		{
			EventRecord theEvent;
			WaitNextEvent(everyEvent, &theEvent, 0, nil);
			result = QTIsStandardParameterDialogEvent(&theEvent, createdDialog);
			switch (result)
			{
				case featureUnsupported:
				{
					result = noErr;
					switch (theEvent.what)
					{
						case updateEvt:
							BeginUpdate((WindowPtr) theEvent.message);
							EndUpdate((WindowPtr) theEvent.message);
							break;
					}
				};break;
				case codecParameterDialogConfirm:
				case userCanceledErr:
				{
					QTDismissStandardParameterDialog(createdDialog);
					createdDialog = nil;
				};break;
			}
		}
	}
	else
	{
						// create the effect ID atom: the atom type is kParameterWhatName, and the atom ID is kParameterWhatID
		myType = kBlurImageFilterType;
		myType = EndianU32_NtoB(kBlurImageFilterType);
		res = QTInsertChild(myEffectDesc, kParentAtomIsContainer, kParameterWhatName, kParameterWhatID, 0, sizeof(myType), &myType, NULL);
	    ThrowOnOSStatus(res);

		long hairDensity = 15;
		res = QTInsertChild(myEffectDesc, kParentAtomIsContainer, 'ksiz', 1, 0, sizeof(hairDensity), &hairDensity, NULL);
	    ThrowOnOSStatus(res);

		Fixed hairDensityd = FixRatio (10,1);
		res = QTInsertChild(myEffectDesc, kParentAtomIsContainer, 'ksum', 1, 0, sizeof(hairDensityd), &hairDensityd, NULL);
	    ThrowOnOSStatus(res);
	}

	// add the first source
	myType = EndianU32_NtoB(kSourceOneName);
	res = QTInsertChild(myEffectDesc, kParentAtomIsContainer, kEffectSourceName, 1, 0, sizeof(myType), &myType, NULL);
    ThrowOnOSStatus(res);

	// add the second source
	if (nbTrackSource == 2)
	{
		myType = EndianU32_NtoB(kSourceTwoName);
		res = QTInsertChild(myEffectDesc, kParentAtomIsContainer, kEffectSourceName, 2, 0, sizeof(myType), &myType, NULL);
	    ThrowOnOSStatus(res);
	}
	
	short index;
	QTAtom atom = QTFindChildByID(myEffectDesc, kParentAtomIsContainer, kParameterWhatName, kParameterWhatID, &index);
	OSTypePtr effectType;
	long size = sizeof(effectType);

	QTLockContainer(myEffectDesc);
	res = QTGetAtomDataPtr(myEffectDesc, atom, &size, (char**)&effectType);
	ThrowOnOSStatus(res);	
	QTUnlockContainer(myEffectDesc);
	
	// creation d'un sampleDescription
	//	ImageDescriptionHandle		mySampleDesc = NULL;

	res = MakeImageDescriptionForEffect(*effectType, &mySampleDesc);
    ThrowOnOSStatus(res);

	(**mySampleDesc).vendor = kAppleManufacturer;
	(**mySampleDesc).temporalQuality = codecNormalQuality;
	(**mySampleDesc).spatialQuality = codecNormalQuality;
	(**mySampleDesc).width = widthSource >> 16;
	(**mySampleDesc).height = heightSource >> 16;	

	// compute the duration of the effect track
	TimeValue endTime = GetTrackDuration(trackSource[0]);
	TimeValue startTime = GetTrackOffset(trackSource[0]);
	
	for (int i=1; i < nbTrackSource; ++i)
	{
		if (GetTrackOffset(trackSource[i]) < startTime)
			startTime = GetTrackOffset(trackSource[i]);
		if (GetTrackDuration(trackSource[i]) < endTime)
			endTime = GetTrackDuration(trackSource[i]);
	}

	TimeValue duration = endTime - startTime;

	// add the effect description as a sample to the effect track media
	TimeValue mySampleTime;

	res = BeginMediaEdits(mediaEffect);
    ThrowOnOSStatus(res);
	res = AddMediaSample(mediaEffect, (Handle)myEffectDesc, 0, GetHandleSize((Handle)myEffectDesc), duration, (SampleDescriptionHandle)mySampleDesc, 1, 0, &mySampleTime);
    ThrowOnOSStatus(res);
//	res += AddMediaSample(mediaEffect, (Handle)myEffectDesc, 0, GetHandleSize((Handle)myEffectDesc), TimeValueFromSec(movieSource, 2), (SampleDescriptionHandle)mySampleDesc, 1, 0, &mySampleTime);

	res = EndMediaEdits(mediaEffect);
    ThrowOnOSStatus(res);

	// create the input map and add references for the first effect track
	res = QTNewAtomContainer(&myInputMap);
	ThrowOnOSStatus(res);	


	QTAtom				myInputAtom;
	long				myRefIndex;

	// first track
	res = AddTrackReference(trackEffect, trackSource[0], kTrackModifierReference, &myRefIndex);
    ThrowOnOSStatus(res);
	
	// add a reference atom to the input map
	res = QTInsertChild(myInputMap, kParentAtomIsContainer, kTrackModifierInput, myRefIndex, 0, 0, NULL, &myInputAtom);
    ThrowOnOSStatus(res);
	
	// add two child atoms to the parent reference atom
	myType = EndianU32_NtoB(kTrackModifierTypeImage);
	res = QTInsertChild(myInputMap, myInputAtom, kTrackModifierType, 1, 0, sizeof(myType), &myType, NULL);
    ThrowOnOSStatus(res);

	myType = EndianU32_NtoB(kSourceOneName);
	res = QTInsertChild(myInputMap, myInputAtom, kEffectDataSourceType, 1, 0, sizeof(myType), &myType, NULL);
    ThrowOnOSStatus(res);

	// second track
	if (nbTrackSource >= 2)
	{
		res = AddTrackReference(trackEffect, trackSource[1], kTrackModifierReference, &myRefIndex);
		ThrowOnOSStatus(res);
		
		// add a reference atom to the input map
		res = QTInsertChild(myInputMap, kParentAtomIsContainer, kTrackModifierInput, myRefIndex, 0, 0, NULL, &myInputAtom);
	    ThrowOnOSStatus(res);
	
		// add two child atoms to the parent reference atom
		myType = EndianU32_NtoB(kTrackModifierTypeImage);
		res = QTInsertChild(myInputMap, myInputAtom, kTrackModifierType, 1, 0, sizeof(myType), &myType, NULL);
	    ThrowOnOSStatus(res);

		myType = EndianU32_NtoB(kSourceTwoName);
		res = QTInsertChild(myInputMap, myInputAtom, kEffectDataSourceType, 1, 0, sizeof(myType), &myType, NULL);
	    ThrowOnOSStatus(res);
	}

   		// add the input map to the effects track
	res = SetMediaInputMap(mediaEffect, myInputMap);
    ThrowOnOSStatus(res);

	// add the media to the track
	res = InsertMediaIntoTrack(trackEffect, 0, mySampleTime, GetMediaDuration(mediaEffect), fixed1);
    ThrowOnOSStatus(res);

//	res = UpdateMovieResource (movieSource, resFileSource, resIdSource, pfSource.fName);	

	if (mySampleDesc != NULL)
		DisposeHandle((Handle)mySampleDesc);
	
	if (myInputMap != NULL)
		QTDisposeAtomContainer(myInputMap);

	vernl(6, "CreateEffectOnMovie : Final res = " << res);

	//move the trackEffect
	SetTrackOffset(trackEffect, startTime);

	return trackEffect;
}


Track CreateTrackForEffect(Movie movieSource, Track trackA, Track trackB, QTAtomContainer& effectDesc)
{
	Track trackSource[2];
	Media mediaSource;
	int nbTrackSource;
	TimeScale timeScaleSource;
	Fixed widthSource, heightSource;
	OSErr  res = noErr;
	ImageDescriptionHandle		mySampleDesc = NULL;

	trackSource[0] = trackA;
	if (trackB)
	{
		trackSource[1] = trackB;
		nbTrackSource = 2;
	}
	else
		nbTrackSource = 1;

	// prise info sur le track video source
	mediaSource = GetTrackMedia(trackSource[0]);
	timeScaleSource = GetMediaTimeScale(mediaSource);
	GetTrackDimensions(trackSource[0], &widthSource, &heightSource);

	// creation de la piste d'effet
	Track trackEffect;
	Media mediaEffect;

    trackEffect = NewMovieTrack(movieSource, widthSource, heightSource, kNoVolume);
	mediaEffect = NewTrackMedia(trackEffect, VideoMediaType, timeScaleSource, NULL, 0); 

	// creation d'un effectDescription
	QTAtomContainer		myInputMap = NULL;
	
	short index;
	QTAtom atom = QTFindChildByID(effectDesc, kParentAtomIsContainer, kParameterWhatName, kParameterWhatID, &index);
	OSTypePtr effectType;
	long size = sizeof(effectType);

	QTLockContainer(effectDesc);
	res = QTGetAtomDataPtr(effectDesc, atom, &size, (char**)&effectType);
	ThrowOnOSStatus(res);	
	QTUnlockContainer(effectDesc);
	
	// creation d'un sampleDescription
	//	ImageDescriptionHandle		mySampleDesc = NULL;

	res = MakeImageDescriptionForEffect(*effectType, &mySampleDesc);
    ThrowOnOSStatus(res);

	(**mySampleDesc).vendor = kAppleManufacturer;
	(**mySampleDesc).temporalQuality = codecNormalQuality;
	(**mySampleDesc).spatialQuality = codecNormalQuality;
	(**mySampleDesc).width = widthSource >> 16;
	(**mySampleDesc).height = heightSource >> 16;	

	// compute the duration of the effect track
	TimeValue endTime = GetTrackDuration(trackSource[0]);
	TimeValue startTime = GetTrackOffset(trackSource[0]);
	
	for (int i=1; i < nbTrackSource; ++i)
	{
		if (GetTrackOffset(trackSource[i]) < startTime)
			startTime = GetTrackOffset(trackSource[i]);
		if (GetTrackDuration(trackSource[i]) < endTime)
			endTime = GetTrackDuration(trackSource[i]);
	}

	TimeValue duration = endTime - startTime;

	// add the effect description as a sample to the effect track media
	TimeValue mySampleTime;

	res = BeginMediaEdits(mediaEffect);
    ThrowOnOSStatus(res);
	
	res = AddMediaSample(mediaEffect, (Handle)effectDesc, 0, GetHandleSize((Handle)effectDesc), duration, (SampleDescriptionHandle)mySampleDesc, 1, 0, &mySampleTime);
    ThrowOnOSStatus(res);

	res = EndMediaEdits(mediaEffect);
    ThrowOnOSStatus(res);

	// create the input map and add references for the first effect track
	res = QTNewAtomContainer(&myInputMap);
	ThrowOnOSStatus(res);	


	QTAtom				myInputAtom;
	long				myRefIndex;
	OSType				myType;

	// first track
	res = AddTrackReference(trackEffect, trackSource[0], kTrackModifierReference, &myRefIndex);
    ThrowOnOSStatus(res);
	
	// add a reference atom to the input map
	res = QTInsertChild(myInputMap, kParentAtomIsContainer, kTrackModifierInput, myRefIndex, 0, 0, NULL, &myInputAtom);
    ThrowOnOSStatus(res);
	
	// add two child atoms to the parent reference atom
	myType = EndianU32_NtoB(kTrackModifierTypeImage);
	res = QTInsertChild(myInputMap, myInputAtom, kTrackModifierType, 1, 0, sizeof(myType), &myType, NULL);
    ThrowOnOSStatus(res);

	myType = EndianU32_NtoB(kSourceOneName);
	res = QTInsertChild(myInputMap, myInputAtom, kEffectDataSourceType, 1, 0, sizeof(myType), &myType, NULL);
    ThrowOnOSStatus(res);

	// second track
	if (nbTrackSource >= 2)
	{
		res = AddTrackReference(trackEffect, trackSource[1], kTrackModifierReference, &myRefIndex);
	    ThrowOnOSStatus(res);
	
		// add a reference atom to the input map
		res = QTInsertChild(myInputMap, kParentAtomIsContainer, kTrackModifierInput, myRefIndex, 0, 0, NULL, &myInputAtom);
	    ThrowOnOSStatus(res);
	
		// add two child atoms to the parent reference atom
		myType = EndianU32_NtoB(kTrackModifierTypeImage);
		res = QTInsertChild(myInputMap, myInputAtom, kTrackModifierType, 1, 0, sizeof(myType), &myType, NULL);
	    ThrowOnOSStatus(res);

		myType = EndianU32_NtoB(kSourceTwoName);
		res = QTInsertChild(myInputMap, myInputAtom, kEffectDataSourceType, 1, 0, sizeof(myType), &myType, NULL);
	    ThrowOnOSStatus(res);
	}

   		// add the input map to the effects track
	res = SetMediaInputMap(mediaEffect, myInputMap);
    ThrowOnOSStatus(res);

	// add the media to the track
	res = InsertMediaIntoTrack(trackEffect, 0, mySampleTime, GetMediaDuration(mediaEffect), fixed1);
    ThrowOnOSStatus(res);

//	res = UpdateMovieResource (movieSource, resFileSource, resIdSource, pfSource.fName);	

	if (mySampleDesc != NULL)
		DisposeHandle((Handle)mySampleDesc);
	
	if (myInputMap != NULL)
		QTDisposeAtomContainer(myInputMap);

	vernl(6, "CreateTrackForEffect : Final res = " << res);

	//move the trackEffect
	SetTrackOffset(trackEffect, startTime);

	return trackEffect;
}

bool ConfigureEffect(QTAtomContainer& effectDesc, int nbTrackSource, PicHandle posterSrc1, PicHandle posterSrc2)
{
	OSErr res = 0;
	
	QTAtomContainer effectList;
	QTParameterDialog createdDialog;
	OSErr result=noErr;

	//$$todoqt : check if we need to allocate the effect list before (QTNewAtomContainer)
	effectList = GetQTEffectsList(nbTrackSource); //QTGetEffectsList(&effectList, nbTrackSource, nbTrackSource, 0);  
	ThrowOnOSStatus(res);	
	
	ExploreQTAtomContainer(&effectList);
	
//		res = QTGetEffectsList(&effectList, 0, 2, 0);  
	res = QTCreateStandardParameterDialog (effectList, effectDesc, pdOptionsAllowOptionalInterpolations, &createdDialog); 
	ThrowOnOSStatus(res);	

        //insert poster frames into dialog
        if (posterSrc1 != NULL)
        {
            QTParamPreviewRecord myPreviewRecord;
            myPreviewRecord.sourcePicture = posterSrc1;
            myPreviewRecord.sourceID = 1;
            res = QTStandardParameterDialogDoAction(createdDialog, pdActionSetPreviewPicture, &myPreviewRecord);
			ThrowOnOSStatus(res);	
        }
        if (posterSrc2 != NULL)
        {
            QTParamPreviewRecord myPreviewRecord;
            myPreviewRecord.sourcePicture = posterSrc2;
            myPreviewRecord.sourceID = 2;
            res = QTStandardParameterDialogDoAction(createdDialog, pdActionSetPreviewPicture, &myPreviewRecord);
			ThrowOnOSStatus(res);	
        }
        while (result == noErr)
	{
		EventRecord theEvent;
		WaitNextEvent(everyEvent, &theEvent, 0, nil);
		result = QTIsStandardParameterDialogEvent(&theEvent, createdDialog);
		switch (result)
		{
			case featureUnsupported:
			{
				result = noErr;
				switch (theEvent.what)
				{
					case updateEvt:
						BeginUpdate((WindowPtr) theEvent.message);
						EndUpdate((WindowPtr) theEvent.message);
						break;
				}
			};break;
			case codecParameterDialogConfirm:
			{
				QTDismissStandardParameterDialog(createdDialog);
				createdDialog = nil;
			};break;
			case userCanceledErr:
			{
				QTDismissStandardParameterDialog(createdDialog);
				createdDialog = nil;
				return false; // effect has not changed
			};break;
		}
	}

/*	DEBUG code to get the value of tween data (uncomment when needed)
    *****************************************************************
	
	QTLockContainer(effectDesc);
	OSType myType = FOUR_CHAR_CODE('bmul');
	short index;
	Ptr		atomData;
	long 	size;
	float from;
	float to;

	QTAtom atom = QTFindChildByID(effectDesc, kParentAtomIsContainer, myType, 1, &index);
	QTAtom pcntAtom = QTFindChildByID(effectDesc, atom, kTweenData, 1, &index);

	if (pcntAtom == 0)
	{
		// it happens when from and to are equal
		QTGetAtomDataPtr(effectDesc, atom, &size, &atomData);
		from = Fix2X(EndianS32_BtoN(((Fixed*)atomData)[0]));
		to = Fix2X(EndianS32_BtoN(((Fixed*)atomData)[0]));
	}
	else
	{
		QTGetAtomDataPtr(effectDesc, pcntAtom, &size, &atomData);
	
		from = Fix2X(EndianS32_BtoN(((Fixed*)atomData)[0]));
		to = Fix2X(EndianS32_BtoN(((Fixed*)atomData)[1]));
	}
	QTUnlockContainer(effectDesc);
*/	
	QTDisposeAtomContainer(effectList);
	return true; // effect has been changed
}

void GetFromAndToParamOfTransition(QTAtomContainer& effectDesc, float& from, float& to)
{
	short index;
	Ptr		atomData;
	long 	size;
	
	QTLockContainer(effectDesc);
	QTAtom atom = QTFindChildByID(effectDesc, kParentAtomIsContainer, kParameterUsagePercent, 1, &index);
	QTAtom pcntAtom = QTFindChildByID(effectDesc, atom, kTweenData, 1, &index);

	if (pcntAtom == 0)
	{
		// it happens when from and to are equal
		QTGetAtomDataPtr(effectDesc, atom, &size, &atomData);
		from = Fix2X(EndianS32_BtoN(((Fixed*)atomData)[0]));
		to = Fix2X(EndianS32_BtoN(((Fixed*)atomData)[0]));
	}
	else
	{
		QTGetAtomDataPtr(effectDesc, pcntAtom, &size, &atomData);
	
		from = Fix2X(EndianS32_BtoN(((Fixed*)atomData)[0]));
		to = Fix2X(EndianS32_BtoN(((Fixed*)atomData)[1]));
	}
	QTUnlockContainer(effectDesc);
}

void GetCopyOfTransitionEffect(QTAtomContainer& effectDest, QTAtomContainer& effectSrc, float from, float to)
{
	QTLockContainer(effectSrc);
	QTLockContainer(effectDest);

	//make a copy
	QTCopyAtom(effectSrc, kParentAtomIsContainer, &effectDest);

	OSErr res = 0;
	// delete the reference on the source tracks
	short index;
	QTAtom atom = QTFindChildByID(effectDest, kParentAtomIsContainer, kEffectSourceName, 2, &index);
	if (atom)
	{
		res = QTRemoveAtom(effectDest, atom);
		ThrowOnOSStatus(res);	
	}
	atom = QTFindChildByID(effectDest, kParentAtomIsContainer, kEffectSourceName, 1, &index);
	if (atom)
	{
		res = QTRemoveAtom(effectDest, atom);
		ThrowOnOSStatus(res);	
	}
	
	// delete the tween atom 
	atom = QTFindChildByID(effectDest, kParentAtomIsContainer, kParameterUsagePercent, 1, &index);
	if (atom)
	{
		res = QTRemoveAtom(effectDest, atom);
		ThrowOnOSStatus(res);	
	}
	//and replace it by the new tween

	QTAtom newAtom;
	res = QTInsertChild(effectDest, kParentAtomIsContainer, kParameterUsagePercent, 1, 0, 0, 0, &newAtom);
    ThrowOnOSStatus(res);
	
		/* this atom specifies the format of the tween data */

	OSType tweenType = EndianU32_NtoB(kParameterTypeDataFixed);
	res = QTInsertChild(effectDest, newAtom, kTweenType, 1 , 0, sizeof(OSType), &tweenType, NULL);
	ThrowOnOSStatus(res);	
	/* here's the data for the tween  */
	Fixed percentage[2];

	percentage[0] =  EndianS32_NtoB( X2Fix(from) );	
	percentage[1] =  EndianS32_NtoB( X2Fix(to) );	
	res = QTInsertChild(effectDest, newAtom, kTweenData, 1 , 0, sizeof(long) * 2, &percentage, NULL);
	ThrowOnOSStatus(res);	

	QTUnlockContainer(effectSrc);
	QTUnlockContainer(effectDest);
}


/*int FindTrackStartingAtTime(ArmeTime t, ZzArray<TVideoTrack*> &list)
{
	int i=0;
	while ((i < list.CountItems()) && (list[i]->fStartTime != t))
	{
		i++;
	}
	
	if (i == list.CountItems())
	{
		return -1;
	}

	return i;
}

int GetNbTracksInCommon(ZzArray<TVideoTrack*>& listOfTrackRepi, ZzArray<TVideoTrack*>& listOfTrackRepj, int startSearchIndexInListI, int& firstIndexInListJ)
{
	int i = startSearchIndexInListI;
	bool bFirstTrackFount = false;
	int index = -1;
	int count = 0;
	firstIndexInListJ = -1;
	
	while ((i < listOfTrackRepi.CountItems()) && ((index = FindTrackStartingAtTime(listOfTrackRepi[i]->fStartTime, listOfTrackRepj)) != -1))
	{
		if (!bFirstTrackFount)
		{
			bFirstTrackFount = true;
			firstIndexInListJ = index;
		}
		else
		{
			// make sure the tracks in common are consecutive
			if (index != firstIndexInListJ+count)
				return count;
		}
		i++;
		count++;
	}
	
	return count;
}

int GetNextStartOfGroupOfTrack(int startSearchIndexInListI, ZzArray<TVideoTrack*>& listOfTrackRepi)
{
	int i = startSearchIndexInListI+1;
	while ((i < listOfTrackRepi.CountItems()) && (listOfTrackRepi[i]->fStartTime == listOfTrackRepi[i-1]->fEndTime))
	{
		i++;
	}
	return i;
}

bool IsAtTheEndOfAGroupOfTrack(int index,  ZzArray<TVideoTrack*>& listOfTrackRepj)
{
	if ((index+1) >= listOfTrackRepj.CountItems())
	{
		return true;
	}
	else if (listOfTrackRepj[index]->fEndTime == listOfTrackRepj[index+1]->fStartTime)
	{
		return false;
	}

	return true;
}

bool IsAtTheBeginOfAGroupOfTrack(int index,  ZzArray<TVideoTrack*>& listOfTrackRepj)
{
	if (index == 0)
		return true;
	else if (listOfTrackRepj[index-1]->fEndTime != listOfTrackRepj[index]->fStartTime)
		return true;
	else
		return false;
}*/

void AddNewSplits(TArmeWSObj* fromParent, ArmeTime startTime, ArmeTime endTime)
{
/*	//first, call this function recursively to generate the XFade effects in the hierarchy below
	int32 childrenCount = fromParent->GetChildrenCount();

	for (int i=0; i < childrenCount; ++i)
	{
		//$$Check if also works with xFade Effects
		if (fromParent->GetChild(i)->IsVideoObject())
		{
			TVideoObject* obj = dynamic_cast<TVideoObject*>(fromParent->GetChild(i));
			if (obj && obj->ContainsVideoDuring(startTime, endTime) && obj->IsVisible())
			{
				AddNewSplits(obj, startTime, endTime);
			}
		}
	}

	ZzArray<TVideoObject*> list;
	ZzArray<ArmeTime> listOfSplitsTime;

	//first, get the list of all the children video object that are defined during this period
	for (int i=0; i < childrenCount; ++i)
	{
		//$$Check if also works with xFade Effects
		if (fromParent->GetChild(i)->IsVideoObject())
		{
			TVideoObject* obj = dynamic_cast<TVideoObject*>(fromParent->GetChild(i));
			if (obj && obj->ContainsVideoDuring(startTime, endTime) && obj->IsVisible())
			{
				if ((obj->GetType() != TArmeWSObj::kVideoXFadeEffect) || ((TVideoXFadeEffect*)obj)->fToBeDeleted) // don't consider XFade effect that are going to be deleted
				{
					list.AddItem(obj);
					obj->AddListOfStartAndEndTimeOfVideoClip(startTime, endTime, listOfSplitsTime);
				}
			}
		}
	}

//	if (list.CountItems() < 2)
//		return;
	
	//then, go through all these objects and add splits at the start and at the end of each of these objects
	for (int i=0; i < list.CountItems(); ++i)
	{
		for (int j=0; j < listOfSplitsTime.CountItems(); ++j)
		{
			list[i]->AddSplit(listOfSplitsTime[j]);
		}
	}

//	RemoveUselessXFadeEffect(fromParent, startTime, endTime);*/
}

TVideoXFadeEffect* FindTransitionBetween(ZzArray<TVideoXFadeEffect*>* transitionsOuttaHierarchy, const TVideoObject* clip1, const TVideoObject* clip2)
{
	for (int i=0; i < transitionsOuttaHierarchy->CountItems(); ++i)
	{
		TVideoXFadeEffect* xFade = transitionsOuttaHierarchy->ItemAt(i);
		if (xFade)
		{
			if ( ((xFade->fClip1 == clip1) && (xFade->fClip2 == clip2))
				|| ((xFade->fClip1 == clip2) && (xFade->fClip2 == clip1))) 
			{
				return xFade;
			}
		}
	}

	return NULL;
}


void CreateXFadeEffect(TArmeWSObj* fromParent, bool bConstantMixTransition, ArmeTime startTime, ArmeTime endTime, ZzArray<TVideoXFadeEffect*>* transitionsOuttaHierarchy, ZzArray<TVideoXFadeEffect*>* transitionsCreated, ZzArray<TVideoXFadeEffect*>* transitionsToDelete, ZzArray<TVideoXFadeEffect*>* transitionsToUpdate)
{
/*	int32 childrenCount = fromParent->GetChildrenCount();

	ZzArray<TVideoObject*> list;

	//first, get the list of all the children video object that are defined during this period
	for (int i=0; i < childrenCount; ++i)
	{
		if (fromParent->GetChild(i)->IsVideoObject())
		{
			TVideoObject* obj = dynamic_cast<TVideoObject*>(fromParent->GetChild(i));
			if (obj && obj->ContainsVideoDuring(startTime, endTime) && obj->IsVisible())
			{
				if (obj->GetType() == TArmeWSObj::kVideoXFadeEffect)
				{
					// don't consider a xFade effect that is going to be deleted
					if (((TVideoXFadeEffect*)obj)->fToBeDeleted == false)
						list.AddItem(obj);
				}
				else
					list.AddItem(obj);
			}
		}
	}

	//then, go through all these objects and generate the XFade effects
	for (int i=0; i < list.CountItems(); ++i)
	{
		for (int j=i+1; j < list.CountItems(); ++j)
		{
			ZzArray<TVideoTrack*> listOfTrackRepi(5);
			ZzArray<TVideoTrack*> listOfTrackRepj(5);
		
			list[i]->AddListOfTrackRep(listOfTrackRepi);
			list[j]->AddListOfTrackRep(listOfTrackRepj);
			
			int startSearchIndexInListI = 0;
			
			int firstTrackInListJ = -1;
			
			while (startSearchIndexInListI < listOfTrackRepi.CountItems())
			{
				int nbTracks = GetNbTracksInCommon(listOfTrackRepi, listOfTrackRepj, startSearchIndexInListI, firstTrackInListJ);
				if (nbTracks) // we will create some xFade effect
				{
					bool effectCreatedNow = false;
		
					// if a transition with this clip has already been configured, use the param from this transition
					TVideoXFadeEffect* xFade = ((TVideoObject*)fromParent)->FindTransitionBetweenChildren(list[i], list[j]);
					if (!xFade) //search in the transitions outta hierarchy
					{
						TVideoXFadeEffect* oldxFade = FindTransitionBetween(transitionsOuttaHierarchy, list[i], list[j]);
						if (oldxFade)
						{
							//create a new effect with the same properties as the one that was outta hierarchy
							//Note : I'm not reusing the effect outta hierarchy because the clips may have moved
							// and if I putBackInHierarchy the effect, it might not be at the correct position
							xFade = new TVideoXFadeEffect(fromParent->GetDoc(), oldxFade->fEffectDesc, oldxFade->fTransitionType); 
							
							//remove old effect
							transitionsOuttaHierarchy->RemoveItemByValue(oldxFade);
							transitionsCreated->AddItem(xFade);
							
							effectCreatedNow = true;
						}
					}
				
					// create a transition from j to i
					if ((IsAtTheBeginOfAGroupOfTrack(startSearchIndexInListI, listOfTrackRepi) && !bConstantMixTransition) && 
						(IsAtTheEndOfAGroupOfTrack(firstTrackInListJ+nbTracks-1, listOfTrackRepj) && !bConstantMixTransition))
					{
						//if no effect was found, create a new one
						if (!xFade)
						{
							xFade = new TVideoXFadeEffect(fromParent->GetDoc(), TVideoXFadeEffect::INCREASE, TVideoXFadeEffect::END, 1.0f, 0.0f);
							transitionsCreated->AddItem(xFade);
						}
						else if (!effectCreatedNow)
						{
							xFade->DeleteEffect();
							transitionsToUpdate->AddItem(xFade);
						}

						if (xFade->fTransitionType != TVideoXFadeEffect::END)
						{
							xFade->fFrom = 1.0f;
							xFade->fTo = 0.0f;
							xFade->fTransitionType = TVideoXFadeEffect::END;
						}

						xFade->fClip1 = list[i];
						xFade->fClip2 = list[j];

						xFade->UpdateEffect();

						// insert the effect in the hierarchy
						TArmeWSObjPos pos;
						pos.fParent = fromParent;
						pos.fStartTime = xFade->GetTrack(0)->fStartTime - fromParent->GetAbsoluteStartTime();
						xFade->_SetPos(pos);
						xFade->_SetLength(xFade->GetLastVideoTrack()->fEndTime - pos.fStartTime);
					}
					else if ((IsAtTheEndOfAGroupOfTrack(startSearchIndexInListI+nbTracks-1, listOfTrackRepi) && !bConstantMixTransition) && 
						(IsAtTheBeginOfAGroupOfTrack(firstTrackInListJ, listOfTrackRepj) && !bConstantMixTransition))
					{ 
						// create a smooth transition
						//if no effect was found, create a new one
						if (!xFade)
						{
							xFade = new TVideoXFadeEffect(fromParent->GetDoc(), TVideoXFadeEffect::INCREASE, TVideoXFadeEffect::BEGIN, 0.0f , 1.0f);
							transitionsCreated->AddItem(xFade);
						}
						else if (!effectCreatedNow)
						{
							xFade->DeleteEffect();
							transitionsToUpdate->AddItem(xFade);
						}
						
						if (xFade->fTransitionType != TVideoXFadeEffect::BEGIN)
						{
							xFade->fFrom = 0.0f;
							xFade->fTo = 1.0f;
							xFade->fTransitionType = TVideoXFadeEffect::BEGIN;
						}

						xFade->fClip1 = list[i];
						xFade->fClip2 = list[j];
						xFade->UpdateEffect();

						// insert the effect in the hierarchy
						TArmeWSObjPos pos;
						pos.fParent = fromParent;
						pos.fStartTime = xFade->GetTrack(0)->fStartTime - fromParent->GetAbsoluteStartTime();
						xFade->_SetPos(pos);
						xFade->_SetLength(xFade->GetLastVideoTrack()->fEndTime - pos.fStartTime);
					}
					else
					{ // create a constant transition
						//if no effect was found, create a new one
						if (!xFade)
						{
							xFade = new TVideoXFadeEffect(fromParent->GetDoc(), TVideoXFadeEffect::CONSTANT, TVideoXFadeEffect::MIDDLE, 0.5f, 0.5f);
							transitionsCreated->AddItem(xFade);
						}
						else if (!effectCreatedNow)
						{
							xFade->DeleteEffect();
							transitionsToUpdate->AddItem(xFade);
						}

						if (xFade->fTransitionType != TVideoXFadeEffect::MIDDLE)
						{
							xFade->fFrom = 0.5f;
							xFade->fTo = 0.5f;
							xFade->fTransitionType = TVideoXFadeEffect::MIDDLE;
						}

						xFade->fClip1 = list[i];
						xFade->fClip2 = list[j];

						xFade->UpdateEffect();

						// insert the effect in the hierarchy
						TArmeWSObjPos pos;
						pos.fParent = fromParent;
						pos.fStartTime = xFade->GetTrack(0)->fStartTime - fromParent->GetAbsoluteStartTime();
						xFade->_SetPos(pos);
						xFade->_SetLength(xFade->GetLastVideoTrack()->fEndTime - pos.fStartTime);
					}
					startSearchIndexInListI += nbTracks;
				}
				else
					startSearchIndexInListI++;
				//have a look at the next group of track in this list
//				startSearchIndexInListI = GetNextStartOfGroupOfTrack(startSearchIndexInListI, listOfTrackRepi);
			}
		}
	}
*/
#ifdef VERBOSE_VIDEO_OBJECT
/*	int childrenCount = TLC->GetChildrenCount();
	for (int i=0; i < childrenCount; ++i)
	{
		if ((TLC->GetChild(i)->GetType() == TArmeWSObj::kVideoEffect)
			|| (TLC->GetChild(i)->GetType() == TArmeWSObj::kVideoClip)
			|| (TLC->GetChild(i)->GetType() == TArmeWSObj::kVideoXFadeEffect))
		{
			TVideoObject* obj = (TVideoObject*)(TLC->GetChild(i));
			obj->Dump();
		}
	}*/
#endif //VERBOSE_VIDEO_OBJECT
}

/*void AddVideoChildrenToTheList(TArmeWSObj* object, ArmeTime startTime, ArmeTime endTime, ZzArray<TVideoObject*> &list)
{
	if (object->IsVideoObject())
	{
		if (((TVideoObject*)object)->ContainsVideoDuring(startTime, endTime))
		{
			list.AddItem((TVideoObject*)object);
		}
	}
	else if (object->GetType() == TArmeWSObj::kTLC)
	{
		int32 childrenCount = object->GetChildrenCount();
		for (int i=0; i < childrenCount; ++i)
		{
			AddVideoChildrenToTheList(object->GetChild(i), startTime, endTime, list);
		}
	}
}*/

void RemoveUselessSplits(TArmeWSObj* fromParent, ArmeTime startTime, ArmeTime endTime)
{
/*	int32 childrenCount = fromParent->GetChildrenCount();
	ZzArray<ArmeTime> listOfUsefulSplitTime(20);
	
	for (int i=0; i < childrenCount; ++i)
	{
		if (fromParent->GetChild(i)->IsVideoObject())
		{
			TVideoObject* obj = dynamic_cast<TVideoObject*>(fromParent->GetChild(i));
			if (obj && obj->ContainsVideoDuring(startTime, endTime) && obj->IsVisible())
			{
				obj->AddListOfStartAndEndTimeOfVideoClip(startTime, endTime, listOfUsefulSplitTime);
			}
		}
	}

	for (int i=0; i < childrenCount; ++i)
	{
		if (fromParent->GetChild(i)->IsVideoObject())
		{
			TVideoObject* obj = dynamic_cast<TVideoObject*>(fromParent->GetChild(i));
			if (obj && obj->ContainsVideoDuring(startTime, endTime))
			{
				obj->RemoveUselessSplitsInClip(startTime, endTime, listOfUsefulSplitTime);
			}
		}
	}

	//delete all unnecessary XFade effect 
	RemoveUselessXFadeEffect(fromParent, startTime, endTime);
*/

#ifdef VERBOSE_VIDEO_OBJECT
/*	childrenCount = fromParent->GetChildrenCount();
	for (int i=0; i < childrenCount; ++i)
	{
		if ((fromParent->GetChild(i)->GetType() == TArmeWSObj::kVideoEffect)
			|| (fromParent->GetChild(i)->GetType() == TArmeWSObj::kVideoClip)
			|| (fromParent->GetChild(i)->GetType() == TArmeWSObj::kTextClip)
			|| (fromParent->GetChild(i)->GetType() == TArmeWSObj::kVideoXFadeEffect))
		{
			TVideoObject* obj = (TVideoObject*)(fromParent->GetChild(i));
			obj->Dump();
		}
	}*/
#endif //VERBOSE_VIDEO_OBJECT
}


void RemoveUselessXFadeEffect(TArmeWSObj* fromParent, ArmeTime startTime, ArmeTime endTime)
{
//	ZzArray<TVideoObject*> list;
	ZzArray<TVideoXFadeEffect*> listXFade;

	//first, get the list of all video object that are defined during this period
	int32 childrenCount = fromParent->GetChildrenCount();

	for (int i=0; i < childrenCount; ++i)
	{
		if (fromParent->GetChild(i)->IsVideoObject())
		{
			TVideoObject* obj = dynamic_cast<TVideoObject*>(fromParent->GetChild(i));
			if (obj && obj->ContainsVideoDuring(startTime, endTime))
			{
				RemoveUselessXFadeEffect(obj, startTime, endTime);
			}
		}
	}

	for (int i=0; i < childrenCount; ++i)
	{
		if (fromParent->GetChild(i)->GetType() == TArmeWSObj::kVideoXFadeEffect)
		{
			TVideoXFadeEffect* obj = (TVideoXFadeEffect*)(fromParent->GetChild(i));
			if (obj->ContainsVideoDuring(startTime, endTime))
			{
				listXFade.AddItem(obj);
			}
		}
	}
	
	//then, go through all these objects 
	for (int x=0; x < listXFade.CountItems(); ++x)
	{
		// if one of the track reference by the xFade effect has moved, delete it
//		if ((listXFade[x]->fTrack1->fStartTime != listXFade[x]->GetFirstTrack()->fStartTime)
//		|| (listXFade[x]->fTrack2->fStartTime != listXFade[x]->GetFirstTrack()->fStartTime))
/*		if ((listXFade[x]->fTrack2 == NULL)
		  || (listXFade[x]->fTrack1 == NULL))
		{
			fromParent->_RemoveChild(listXFade[x]);
			delete listXFade[x];
		}*/
	}
}

Rect ScaleRectToRect(const Rect& inSourceRect, const Rect& inDestRect, bool inCenter)
{
    // make a Rect to output, same as the source
    Rect outRect = inSourceRect;

    // make a Rect to map from
    Rect mapFromRect = outRect;
    
    // fix the ratio of the destination
    Rect mapToRect = inDestRect;
    double inDestRectwidth = inDestRect.right - inDestRect.left;
    double inSourceRectwidth = inSourceRect.right - inSourceRect.left;
    double inDestRectheight = inDestRect.bottom - inDestRect.top;
    double inSourceRectheight = inSourceRect.bottom - inSourceRect.top;
    double xRatio = inDestRectwidth / inSourceRectwidth;
    double yRatio = inDestRectheight / inSourceRectheight;
    if (xRatio > yRatio)
        mapToRect.right = mapToRect.left + (short)(yRatio * inSourceRectwidth);
    else
        mapToRect.bottom = mapToRect.top + (short)(xRatio * inSourceRectheight);
    
    // and map to ratio-corrected destination
    ::MapRect(&outRect, &mapFromRect, &mapToRect);

    if (inCenter)
    {
	long 	resWidth = outRect.right - outRect.left;
	long 	resHeight = outRect.bottom - outRect.top;
	::OffsetRect(&outRect, 
                inDestRect.left + ((long)inDestRectwidth - resWidth) / 2 - outRect.left, 
		inDestRect.top + ((long)inDestRectheight - resHeight) / 2 - outRect.top);
    }
    
    return outRect;
}

OSStatus FSpPathFromLocation(const FSSpec *spec, char *fullPath)
{
    fullPath[0] = 0;
    
    FSRef ref;
    ::FSpMakeFSRef(spec, &ref);
    
    OSStatus err = ::FSRefMakePath(&ref, (unsigned char *)fullPath, kMaxPathLength - 1 );

    return err;
 }

// Benjamin's suggested temp file naming code
// If anyone knows how to make an FSSpec with > 31 characters in the name, then we could use this for Carbon API calls
char* CreateTempMoviePath(char* projectPath) // <--- project path always ends with a trailing slash (at least it should !!!)
{
	char* tempMoviePath = new char[kMaxPathLength+1]; // <--- defined in ArmeBasicTypes.h. you can move it in ZzTypes if you want
	char uuid[37];
	ZzFill37StringWithGUID(uuid, 37); // maxLen should take the trailing NUL into account
	ZzAssert(strlen(projectPath)+36 <= kMaxPathLength);
	sprintf(tempMoviePath,"%sMOVIE%s.mov",projectPath,uuid);
	vvernl(6,tempMoviePath);
	return tempMoviePath; // caller is responsible for freeing the char buffer, as in all functions that start with "Create"
}

// "ViaFSRef" documents that path's target must exist or this will fail
OSStatus Path2FSSpecViaFSRef(FSSpec& outSpec, const char *inPath, Boolean inIsDirectory)
{
    if (!inPath) 
        return paramErr;

	FSRef ref;
	OSStatus err = ::FSPathMakeRef((const UInt8*)inPath, &ref, &inIsDirectory);
    if (!err) 
		err = ::FSGetCatalogInfo(&ref, kFSCatInfoNone, NULL, NULL, &outSpec, NULL);
    
    return err;
}

// A guaranteed-unique-FSSpec maker, expecting template of "description <%ld>.extension" form
OSStatus MakeUniqueFSSpec(FSSpec& outSpec, const char* inProjectPath, const char* inNameTemplate)
{
	ZzAssert(strlen(inNameTemplate) < (31 - 10 + 3)); // max FSSpec name length - strlen(2^32) + strlen "%ld"
	
	OSStatus err = Path2FSSpecViaFSRef(outSpec, inProjectPath, true);
	if (err)
		return err;
	
	int tryCount = 0;
	do
	{
		if (tryCount)
		{
			if (tryCount > 25) // something is *really* odd -- path length would be a good place to check first
				return abortErr;
			UInt32 whatever;
			::Delay(1, &whatever);
		}
		tryCount++;
	
		char name[256];
		sprintf(name, inNameTemplate, ::TickCount());
		c2pstrcpy(outSpec.name, name);
		err = ::FSMakeFSSpec(outSpec.vRefNum, outSpec.parID, outSpec.name, &outSpec);
	}
	while (err == noErr); // aka "this file exists"
	
	return err == fnfErr ? noErr : err;
}

extern int CountClippableTracks(const Movie inMovie);
int CountClippableTracksInFile(const char* path)
{
    // find and open the movie
    FSSpec movieFile;
	if (noErr != Path2FSSpecViaFSRef(movieFile, path, false))
		return 0;
   
	short movieRef;
    short itsID = DoTheRightThing;
    Movie movie = nil;
    OSErr qtErr = ::OpenMovieFile(&movieFile, &movieRef, fsRdPerm);
    
	if (!qtErr)
        qtErr = ::NewMovieFromFile(&movie, movieRef, &itsID, nil, 0 , nil);
    ::CloseMovieFile(movieRef);
    
	if (qtErr or !movie)
		return 0;
    
    int nb = CountClippableTracks(movie);
	DisposeMovie(movie);
	return nb;
}

void	ConvertMovie(const char* pathSrc, char* pathDest, WorkingCodecType type)
{
	FSSpec dest;
	
	ArmeErr err;


    // find and open the movie
    FSSpec movieFile;
	if (noErr != Path2FSSpecViaFSRef(movieFile, pathSrc, false))
	{
		err.type = kArmeCannotCreateFSSpecFromPath;
		return;
	}
   
	short movieRef;
    short itsID = DoTheRightThing;
    Movie movie = nil;
    OSErr qtErr = ::OpenMovieFile(&movieFile, &movieRef, fsRdPerm);
    
	if (!qtErr)
        qtErr = ::NewMovieFromFile(&movie, movieRef, &itsID, nil, 0 , nil);
    ::CloseMovieFile(movieRef);
	
//	Path2FSSpecViaFSRef(src, pathSrc, false);
	dest = FSSpecFromNonExistingFilePath(pathDest, FOUR_CHAR_CODE('TVOD'), FOUR_CHAR_CODE('MooV'), qtErr);
	//Movie movie = LoadMovie(pathSrc, &movieResFile, &movieResId, &err, &src, false);


/*	OSErr res = ConvertFileToMovieFile(&src, &dest, FOUR_CHAR_CODE('TVOD'), smCurrentScript, 0, createMovieFileDeleteCurFile | movieToFileOnlyExport | movieFileSpecValid | showUserSettingsDialog
, 0, (MovieProgressUPP)-1, 0);
*/

	OSErr res = ConvertMovieToFile(movie, 0, &dest, 0, 0, -1, 0, 0, GetCodecInstance(type)); 
	ThrowOnOSStatus(res);	
}

ComponentInstance 	GetCodecInstance()
{
	ComponentInstance exportCodec = NULL;
	exportCodec = OpenDefaultComponent(StandardCompressionType, StandardCompressionSubType);

	OSErr res = SCRequestSequenceSettings(exportCodec);
	if (res == scUserCancelled)
		return NULL;

	return exportCodec;
}

ComponentInstance	GetCodecInstance(WorkingCodecType type)
{
	ComponentDescription cd;
	Component c = 0;
	OSErr res;

	ComponentInstance exportCodec = NULL;

	switch (type)
	{
		case kDvPal720_576 :
		case kDvPal16_9 :
		{
			cd.componentType = MovieExportType;
			cd.componentSubType = kQTFileTypeDVC;
			cd.componentManufacturer = kAppleManufacturer;
			cd.componentFlags = movieExportMustGetSourceMediaType;
			cd.componentFlagsMask = movieExportMustGetSourceMediaType;
			c = FindNextComponent( c, &cd );
		
			if (c != nil)
			{
				exportCodec = OpenComponent(c);

				/* QTAtomTypes for atoms in import/export settings containers*/
				//  kQTSettingsDVExportNTSC       = 'dvcv', /* True is export as NTSC, false is export as PAL. (Boolean)*/
				QTAtomContainer exportSettings;
				QTAtom videoAtom;
				QTAtom palAtom;
				bool palSettings = false;
				
				res = MovieExportGetSettingsAsAtomContainer(exportCodec, &exportSettings);
				ThrowOnOSStatus(res);	
			
				videoAtom = QTFindChildByID(exportSettings, kParentAtomIsContainer, kQTSettingsVideo, 1, NULL);

				
				palAtom = QTFindChildByID(exportSettings, videoAtom, kQTSettingsDVExportNTSC, 1, NULL);
				if (palAtom != 0)
//					res = QTRemoveAtom(exportSettings, palAtom);
					res = QTSetAtomData(exportSettings, palAtom, 1, &palSettings);
				else
					res = QTInsertChild(exportSettings, videoAtom, kQTSettingsDVExportNTSC, 0, 0, 1, &palSettings, &palAtom);
				ThrowOnOSStatus(res);
				
/*				ExploreQTAtomContainer(&exportSettings);	*/
				res = MovieExportSetSettingsFromAtomContainer(exportCodec, exportSettings);
				ThrowOnOSStatus(res);
				
				return exportCodec;
			}
		};break;
		case kDvNtsc720_480 :
		case kDvNtsc16_9 :
		{
			cd.componentType = MovieExportType;
			cd.componentSubType = kQTFileTypeDVC;
			cd.componentManufacturer = kAppleManufacturer;
			cd.componentFlags = movieExportMustGetSourceMediaType;
			cd.componentFlagsMask = movieExportMustGetSourceMediaType;
			c = FindNextComponent( c, &cd );
		
			if (c != nil)
			{
				exportCodec = OpenComponent(c);

/*				QTAtomContainer exportSettings;
				res = MovieExportGetSettingsAsAtomContainer(exportCodec, &exportSettings);
				ExploreQTAtomContainer(&exportSettings);*/

				return exportCodec;
			}
		};break;
		case kMjpeg320_240 :
		case kMjpeg160_120 :
		case kPictureHighQuality720_576 :
		case kPictureHighQuality720_480 :
		case kPictureHighQuality320_240 :
		case kPictureHighQuality160_120 :
		case kExportDvPal720_576 :
		case kExportDvNtsc720_480 : 
		{
			cd.componentType = MovieExportType;
			cd.componentSubType = MovieFileType;
			cd.componentManufacturer = 0;
			cd.componentFlags = 0;
			cd.componentFlagsMask = 0;
			c = FindNextComponent( c, &cd );
		
			if (c != nil)
			{
				exportCodec = OpenComponent(c);

				SCTemporalSettings temporalSetting;
				SCSpatialSettings spatialSetting;
/*				sc = OpenDefaultComponent(StandardCompressionType, StandardCompressionSubType);
				ss.codecType=EndianU32_NtoB(kIndeo4CodecType);
				ss.codec=0;
				ss.depth=EndianU16_NtoB(24);
				ss.spatialQuality = EndianU32_NtoB(codecLosslessQuality);
				res = SCSetInfo(sc, scSpatialSettingsType, &ss);
				res = SCGetSettingsAsAtomContainer(sc, &compressorData);				

				res = MovieExportSetSettingsFromAtomContainer(exportCodec, compressorData);
				
				return exportCodec;
*/								
				QTAtomContainer exportSettings;
				QTAtom videoAtom;
				QTAtom sizeAtom;
				QTAtom sptlAtom;
				QTAtom tprlAtom;
				Fixed height, width;
				
				res = MovieExportGetSettingsAsAtomContainer(exportCodec, &exportSettings);
				ThrowOnOSStatus(res);
						
				ExploreQTAtomContainer(&exportSettings);
				
				videoAtom = QTFindChildByID(exportSettings, kParentAtomIsContainer, kQTSettingsVideo, 1, NULL);
				// Find the 'sptl' Atom - Spatial Settings
				sptlAtom = QTFindChildByID(exportSettings, videoAtom, scSpatialSettingsType, 1, NULL);
				// Set up our settings
				if (type == kExportDvNtsc720_480)
				{
					spatialSetting.codecType = EndianU32_NtoB(kDVCNTSCCodecType); 
				}
				else if (type == kExportDvPal720_576)
				{
					spatialSetting.codecType = EndianU32_NtoB(kDVCPALCodecType); 
				}
				else if ((type == kPictureHighQuality720_576) || (type == kPictureHighQuality720_480) || (type == kPictureHighQuality320_240) || (type == kPictureHighQuality160_120))
				{
					spatialSetting.codecType = EndianU32_NtoB(kJPEGCodecType); 
				}
				else 
				{
					spatialSetting.codecType = EndianU32_NtoB(kMotionJPEGACodecType); 
				}
				spatialSetting.codec = 0;
				spatialSetting.depth = EndianU16_NtoB(24);
				spatialSetting.spatialQuality = EndianU32_NtoB(codecLosslessQuality);
				// Change the atom data to reflect our settings
				res = QTSetAtomData(exportSettings, sptlAtom, sizeof(spatialSetting), &spatialSetting);
				ThrowOnOSStatus(res);
				// Find the 'tprl' Atom - Temporal Settings
				tprlAtom = QTFindChildByID(exportSettings, videoAtom, scTemporalSettingsType, 1, NULL);
				// Set up our settings
				if ((type == kMjpeg160_120) || (type == kPictureHighQuality160_120))
				{
					temporalSetting.frameRate = EndianU32_NtoB(24 << 16); 
					width = EndianU32_NtoB(160 << 16);
					height = EndianU32_NtoB(120 << 16);
				}
				else if ((type == kMjpeg320_240) || (type == kPictureHighQuality320_240))
				{
					temporalSetting.frameRate = EndianU32_NtoB(30 << 16); 
					width = EndianU32_NtoB(320 << 16);
					height = EndianU32_NtoB(240 << 16);
				}
				else if ((type == kPictureHighQuality720_576) || (type == kExportDvPal720_576))
				{
					temporalSetting.frameRate = EndianU32_NtoB(30 << 16); 
					width = EndianU32_NtoB(720 << 16);
					height = EndianU32_NtoB(576 << 16);
				}
				else if ((type == kPictureHighQuality720_480) || (type == kExportDvNtsc720_480))
				{
					temporalSetting.frameRate = EndianU32_NtoB(FixRatio(2997, 100)); 
					width = EndianU32_NtoB(720 << 16);
					height = EndianU32_NtoB(480 << 16);
				}

				temporalSetting.keyFrameRate = 0;
				temporalSetting.temporalQuality = EndianU32_NtoB(codecLosslessQuality);
				// Change the atom data to reflect our settings
				res = QTSetAtomData(exportSettings, tprlAtom, sizeof(temporalSetting), &temporalSetting);
				ThrowOnOSStatus(res);
					
				// add an atom of type movieExportHeight, or replace data of existing atom
				sizeAtom = QTFindChildByID(exportSettings, videoAtom, movieExportHeight, 1, NULL);
				if (sizeAtom != 0)
					res = QTSetAtomData(exportSettings, sizeAtom, sizeof(height), &height);
				else
					res = QTInsertChild(exportSettings, videoAtom, movieExportHeight, 1, 0, sizeof(height), &height, NULL);
				ThrowOnOSStatus(res);
						
				// add an atom of type movieExportWidth, or replace data of existing atom
				sizeAtom = QTFindChildByID(exportSettings, videoAtom, movieExportWidth, 1, NULL);
				if (sizeAtom != 0)
					res = QTSetAtomData(exportSettings, sizeAtom, sizeof(width), &width);
				else
					res = QTInsertChild(exportSettings, videoAtom, movieExportWidth, 1, 0, sizeof(width), &width, NULL);
				ThrowOnOSStatus(res);	
				res = MovieExportSetSettingsFromAtomContainer(exportCodec, exportSettings);
				ThrowOnOSStatus(res);	
			}		
		};break;
		case kAiff :
		{
			return GetAiffCodecInstance(44100);
		};break;
		default :
			return NULL;
	}

	return exportCodec;
}


ComponentInstance	GetAiffCodecInstance(int sampleRate)
{
	ComponentDescription cd;
	Component c = 0;
	OSErr res;

	ComponentInstance exportCodec = NULL;

	cd.componentType = MovieExportType;
	cd.componentSubType = kQTFileTypeAIFF;
	cd.componentManufacturer = SoundMediaType;
	cd.componentFlags = 0;
	cd.componentFlagsMask = 0;
	c = FindNextComponent( c, &cd );

	if (c != nil)
		exportCodec = OpenComponent(c);
		
	// create a sound sample description
	SoundDescriptionHandle		audioSampleDesc = NULL;
	audioSampleDesc = (SoundDescriptionHandle)NewHandle(sizeof(SoundDescription));
	if (!audioSampleDesc)
	{
		ZZ_THROW(ZzExceptionOutOfMemory);
	}

	//configure the sample rate and format
	(**audioSampleDesc).descSize			= sizeof(SoundDescription);
	(**audioSampleDesc).dataFormat			= kFloat32Format;
	(**audioSampleDesc).resvd1				= 0;
	(**audioSampleDesc).resvd2				= 0;
	(**audioSampleDesc).dataRefIndex		= 0;
	(**audioSampleDesc).version				= 0;
	(**audioSampleDesc).revlevel			= 0;
	(**audioSampleDesc).vendor				= 0;
	(**audioSampleDesc).numChannels			= 2;
	(**audioSampleDesc).sampleSize			= 32;
	(**audioSampleDesc).compressionID		= 0;
	(**audioSampleDesc).packetSize			= 0;
	(**audioSampleDesc).sampleRate			= sampleRate << 16; //fixed

	res = MovieExportSetSampleDescription(exportCodec, (SampleDescriptionHandle)audioSampleDesc, SoundMediaType);
	ThrowOnOSStatus(res);

	return exportCodec;
}

void GetCopyOfEffectDesc(QTAtomContainer& effectDest, QTAtomContainer& effectSrc, float from, float to)
{
	QTLockContainer(effectSrc);
	QTLockContainer(effectDest);

	//make a copy
	QTCopyAtom(effectSrc, kParentAtomIsContainer, &effectDest);
	QTUnlockContainer(effectSrc);

	ExtractEffectDesc(effectDest, kParentAtomIsContainer, from, to);

	QTUnlockContainer(effectDest);
}

void ExtractEffectDesc(QTAtomContainer& effectDesc, QTAtom parentAtom, float from, float to)
{
	//note : it seems that this function works fine only on the first atomContainer you want to analyze.
	// every next call seems to analyze the first atom !!!! except for the 'what' atom that looks correct.
	//example : first analyze a lens flare then a hsl.  then, the hsl has member 'size' .... ?!?

 //   char tempBuffer[512];
	Ptr tempBuffer;
	long sizeWritten;
    
    QTAtomType atomType = 0;
    while (atomType = QTGetNextChildType(effectDesc, parentAtom, atomType))
    {
		short atomCount = QTCountChildrenOfType(effectDesc, parentAtom, atomType);
		for (short i=0; i < atomCount; ++i)
		{
			QTAtom atom = QTFindChildByIndex(effectDesc, parentAtom, atomType, i+1, NULL);
			OSErr res;
			res = QTGetAtomDataPtr(effectDesc, atom, &sizeWritten, &tempBuffer);
			if (res == -2102) //the atom is not a leaf atom
			{
				ExtractEffectDesc(effectDesc, atom, from, to);
			}
			else
			{
				if ((atomType == FOUR_CHAR_CODE('data')) && (sizeWritten == 8))
				{
					float pcntFrom = Fix2X(EndianS32_BtoN(((Fixed*)tempBuffer)[0]));
					float pcntTo = Fix2X(EndianS32_BtoN(((Fixed*)tempBuffer)[1]));
			
					float newPcntFrom = pcntFrom + (pcntTo - pcntFrom)*from;
					float newPcntTo = pcntFrom + (pcntTo - pcntFrom)*to;
					((Fixed*)tempBuffer)[0] = EndianS32_NtoB( X2Fix(newPcntFrom) );	
					((Fixed*)tempBuffer)[1] = EndianS32_NtoB( X2Fix(newPcntTo) );	
				}
			}	
		}
    }
}

QTAtomContainer GetQTEffectsList(int nbSources)
{
	QTAtomContainer effectList;
	QTGetEffectsList(&effectList, nbSources, nbSources, 0);  

	OSType excludeEffects[2] = {FOUR_CHAR_CODE('blnd'), FOUR_CHAR_CODE('chan')};
	int indexToExclude = 0;
	Ptr tempBuffer;
	long sizeWritten;
	OSErr res = noErr;

//	QTLockContainer(effectList);
	for (int iEffectToExclude=0; iEffectToExclude < 2 ; iEffectToExclude++)
	{
		//first, find the index of the effect in the list
		short atomCount = QTCountChildrenOfType(effectList, kParentAtomIsContainer, FOUR_CHAR_CODE('type'));
		for (short i=0; i < atomCount; ++i)
		{
			QTAtom atom = QTFindChildByIndex(effectList, kParentAtomIsContainer, FOUR_CHAR_CODE('type'), i+1, NULL);
			QTGetAtomDataPtr(effectList, atom, &sizeWritten, &tempBuffer);
			if (*(OSType*)(tempBuffer) == excludeEffects[iEffectToExclude])
			{
				indexToExclude = i;
				break;
			}
		}
		
       QTAtomType atomType = 0;
		while (atomType = QTGetNextChildType(effectList, kParentAtomIsContainer, atomType))
		{
			QTAtom atom = QTFindChildByIndex(effectList, kParentAtomIsContainer, atomType, indexToExclude+1, NULL);
			res = QTRemoveAtom(effectList, atom);
			ThrowOnOSStatus(res);
		}
	}
//	QTUnlockContainer(effectList);	
	ExploreQTAtomContainer(&effectList);
	return effectList;
}

bool ExploreQTAtomContainer(QTAtomContainer* atomContainer)
{
	QTLockContainer(*atomContainer);
    ExploreQTAtom(atomContainer, kParentAtomIsContainer);
	QTUnlockContainer(*atomContainer);

	return true;
}

bool ExploreQTAtom(QTAtomContainer* atomContainer, QTAtom parentAtom)
{
	//note : it seems that this function works fine only on the first atomContainer you want to analyze.
	// every next call seems to analyze the first atom !!!! except for the 'what' atom that looks correct.
	//example : first analyze a lens flare then a hsl.  then, the hsl has member 'size' .... ?!?

 //   char tempBuffer[512];
	Ptr tempBuffer;
	long sizeWritten;
    
    QTAtomType atomType = 0;
    while (atomType = QTGetNextChildType(*atomContainer, parentAtom, atomType))
    {
		short atomCount = QTCountChildrenOfType(*atomContainer, parentAtom, atomType);
		for (short i=0; i < atomCount; ++i)
		{
			QTAtom atom = QTFindChildByIndex(*atomContainer, parentAtom, atomType, i+1, NULL);
			OSErr res;
//			res = QTCopyAtomDataToPtr(*atomContainer, atom, TRUE, 512, (void*)tempBuffer, &sizeWritten);
			res = QTGetAtomDataPtr(*atomContainer, atom, &sizeWritten, &tempBuffer);
			if (res == -2102) //the atom is not a leaf atom
			{
/*				WriteSInt32(-1);
				WriteSInt32(atomType);
				WriteSInt32(parentAtom);
				WriteQTAtom(atomContainer, atom);
				WriteSInt32(0);*/
				ExploreQTAtom(atomContainer, atom);
			}
			else
			{
			/*	WriteSInt32(atomType);
				WriteSInt32(parentAtom);
				WriteSInt32(sizeWritten);
				WriteRawData(tempBuffer, sizeWritten);
			*/}	
		}
    }

	return true;
}


//#if defined(_MWERKS_)
pascal 
//endif
OSErr QTMovieProgressProc (Movie theMovie, short message, short whatOperation, Fixed percentDone, long refcon)
{
	try
	{
		MIProgressBarDlgManager* prog = (MIProgressBarDlgManager*)refcon;
		static float lastPcnt = 0.0f;
		
		float pcnt = float(percentDone)/(1 << 16);
	
		if (fabs(lastPcnt - pcnt) > 0.01f)
			prog->Update(pcnt);
	
		return noErr;
	}
	catch (ZzException* e)
	{
		return userCanceledErr;
	}
} 

/*TimeValue TimeValueFromSec(const Movie movie, float sec) 
{
    return (TimeValue)(GetMovieTimeScale(movie)*sec+0.5f);
};

float SecondsFromTimeValue(const Movie movie, TimeValue time)
{
	return (float)time/(float)GetMovieTimeScale(movie);
};*/

TimeValue TimeValueFromArmeTime2(const Movie movie, const TArmeDoc* doc, const ArmeTime time)
{
	TimeValue ret;
	long videoTimeScale = GetMovieTimeScale(movie);
	TimeValue temp1 = 0;
	TimeValue temp2 = 0; 
	TimeValue videoFrameDuration = 0;
	OSType mediaTypes[1];
	mediaTypes[0] = VisualMediaCharacteristic;
	
	GetMovieNextInterestingTime(movie, nextTimeMediaSample, 1, mediaTypes, 0, 0, &temp1, 0);
	GetMovieNextInterestingTime(movie, nextTimeMediaSample, 1, mediaTypes, temp1, 0, &temp2, 0);
	videoFrameDuration = temp2 - temp1;
	
	ret = (time.data * videoTimeScale / doc->GetAudioFrameRate() + videoFrameDuration/2) / videoFrameDuration;
	ret *= videoFrameDuration;
	
	return ret;
}

ArmeTime ArmeTimeFromTimeValue2(const Movie movie, const TArmeDoc* doc, const TimeValue time)
{
	ArmeTime ret;
	SInt64 videoTimeScale = GetMovieTimeScale(movie);
	
	ret.data = SInt64(time) * SInt64(doc->GetAudioFrameRate()) / videoTimeScale;
	
	return ret;
}

float GetMovieDurationSec(const Movie movie) 
{
	return float(GetMovieDuration(movie))/float(GetMovieTimeScale(movie));
};


OSErr CopyChunkListIntoTrack(Track destTrack, TArmeChunkList<TVideoChunk>* chunkList)
{
	OSErr res = noErr;
	
	if (chunkList->GetNbChunks() <= 0)
	{
		ZzAssert(0);
		return res;
	}
	
	TArmeDoc* doc = chunkList->GetChunk(0).fDataFileRef->fDoc;
	TArmeMasterMovie* masterMovie = doc->fMasterMovie;
	ArmeTime curChunkStart = chunkList->GetStartTime();
	
	for (int i=0; i < chunkList->GetNbChunks(); ++i)
	{
		//WARNING don't step over this line, it crashes the debugger !
		const TVideoChunk& chunk = chunkList->GetChunk(i);

		TimeValue duration = doc->TimeValueFromArmeTime(chunk.fDuration);
		
		// if the chunk takes less than one video frame, don't copy it
		if (duration >= doc->fMasterMovie->GetVideoFrameDuration())
		{
			//first, clear the area that we will use
			res = DeleteTrackSegment(destTrack, doc->TimeValueFromArmeTime(curChunkStart), duration);
			ThrowOnOSStatus(res);	

			TimeValue srcStartTime = masterMovie->GetOriginalTrack()->GetStartTimeOfFile((const ArmeVideoDataFileRef*)(chunk.fDataFileRef)) + doc->RoughTimeValueFromArmeTime(chunk.fStartTimeInFile);
			TimeValue srcDuration = doc->RoughTimeValueFromArmeTime(chunk.fDurationInFile);
			TimeValue destStartTime = doc->TimeValueFromArmeTime(curChunkStart);
			
			//make really sure that the chunk does not go further than its source file :
			TimeValue nextFileSrcStartTime = masterMovie->GetOriginalTrack()->GetStartTimeOfNextFile((const ArmeVideoDataFileRef*)(chunk.fDataFileRef));

//			if (srcStartTime + srcDuration >= nextFileSrcStartTime)
			if (srcStartTime + srcDuration > nextFileSrcStartTime)
			{
				srcDuration = nextFileSrcStartTime - doc->fMasterMovie->GetVideoFrameDuration() - srcStartTime;
			}
			
			if (srcDuration > 0)
			{
				//insert the track segment as it is in the original track
				res = InsertTrackSegment(masterMovie->GetOriginalTrack()->GetQTTrack(), destTrack, srcStartTime, srcDuration, destStartTime);
				ThrowOnOSStatus(res);	
				
				//scale the track segment
				if (srcDuration != duration)
				{
					res = ScaleTrackSegment(destTrack, doc->TimeValueFromArmeTime(curChunkStart), srcDuration, duration);
					ThrowOnOSStatus(res);	
				}
			}
		}
	
		curChunkStart += chunkList->GetChunk(i).fDuration;
	}
		
	return res;
}

OSErr CopyEffectDescIntoTrack(TArmeDoc* doc, Track destTrack, QTAtomContainer effectDesc, ArmeTime startTime, ArmeTime endTime)
{
	OSErr res;
	short index;
	QTAtom atom = QTFindChildByID(effectDesc, kParentAtomIsContainer, kParameterWhatName, kParameterWhatID, &index);
	OSTypePtr effectType;
	long size = sizeof(effectType);
	ImageDescriptionHandle		mySampleDesc = NULL;
	
	QTLockContainer(effectDesc);
	res = QTGetAtomDataPtr(effectDesc, atom, &size, (char**)&effectType);
	ThrowOnOSStatus(res);
	QTUnlockContainer(effectDesc);
	
	// create a sampleDescription
	res = MakeImageDescriptionForEffect(*effectType, &mySampleDesc);
	ThrowOnOSStatus(res);
	
	(**mySampleDesc).vendor = kAppleManufacturer;
	(**mySampleDesc).temporalQuality = codecNormalQuality;
	(**mySampleDesc).spatialQuality = codecNormalQuality;
	(**mySampleDesc).width = doc->GetMasterMovie()->GetWidth();
	(**mySampleDesc).height = doc->GetMasterMovie()->GetHeight();	

	TimeValue startValue = doc->TimeValueFromArmeTime(startTime);
	TimeValue duration = doc->TimeValueFromArmeTime(endTime) - startValue;

	// add the effect description as a sample to the effect track media
	Media mediaEffect = GetTrackMedia(destTrack);
	TimeValue mySampleTime = startValue;

	if (duration >= doc->fMasterMovie->GetVideoFrameDuration())
	{
		res = BeginMediaEdits(mediaEffect);
		ThrowOnOSStatus(res);

		res = AddMediaSample(mediaEffect, (Handle)(effectDesc), 0, GetHandleSize((Handle)(effectDesc)), duration, (SampleDescriptionHandle)mySampleDesc, 1, 0, &mySampleTime);
		ThrowOnOSStatus(res);
		res = EndMediaEdits(mediaEffect);
		ThrowOnOSStatus(res);
	
		// delete what was there before
		res = DeleteTrackSegment(destTrack, startValue, duration);
		ThrowOnOSStatus(res);
		// copyt the effect desc in the track
		res = InsertMediaIntoTrack(destTrack, startValue, mySampleTime, duration, 1 << 16);
		ThrowOnOSStatus(res);
	}

	return res;
}

OSErr CopyBackgroundInTrack(TArmeDoc* doc, Track destTrack, ArmeTime startTime, ArmeTime endTime)
{
	//first delete the track where the chunk will be copied
	OSErr res = DeleteTrackSegment(destTrack, doc->TimeValueFromArmeTime(startTime), doc->TimeValueFromArmeTime(endTime - startTime));

	//put a white background in the track
	ArmeErr armeErr;
	short movieResFile;
	short movieResId;

	//note : right now, background is always white
	ZzString zsBundlePath = ZzGetMainBundlePath();
	ZzString zsImagePath = zsBundlePath + "/Contents/Resources/white3232.jpg";
	const char* imagePath = zsImagePath.getUTF8String();

	Movie backgroundMovie = LoadMovie(imagePath, &movieResFile, &movieResId, &armeErr);
	Track backgroundTrack = GetFirstVideoTrack(backgroundMovie);
	
	// !! background movie and destTrack (in mastermovie) can have different timeScale
	ArmeTime backgroundDuration = ArmeTimeFromTimeValue2(backgroundMovie, doc, GetMovieDuration(backgroundMovie));
	
	res = InsertTrackSegment(destTrack, backgroundTrack, 0, GetMovieDuration(backgroundMovie), doc->TimeValueFromArmeTime(startTime));
	ThrowOnOSStatus(res);
	res = ScaleTrackSegment(destTrack, doc->TimeValueFromArmeTime(startTime), doc->TimeValueFromArmeTime(backgroundDuration), doc->TimeValueFromArmeTime(endTime - startTime));
	ThrowOnOSStatus(res);
	DisposeMovie(backgroundMovie);
	
	return res;
}

void GetEffectName(QTAtomContainer& effectDesc, char* dest)
{
	short					myNumEffects;
	short					myIndex;
	OSErr					res;
	QTAtomContainer			myEffectsList = theMovieHelper.GetEffectsList();

	short index;
	QTAtom atom = QTFindChildByID(effectDesc, kParentAtomIsContainer, kParameterWhatName, kParameterWhatID, &index);
	OSTypePtr effectType;
	long size = sizeof(effectType);

	QTLockContainer(effectDesc);
	res = QTGetAtomDataPtr(effectDesc, atom, &size, (char**)&effectType);
	ThrowOnOSStatus(res);
	QTUnlockContainer(effectDesc);
	
	// the returned effects list contains (at least) two atoms for each available effect component,
	// a name atom and a type atom; happily, this list is already sorted alphabetically by effect name
	myNumEffects = QTCountChildrenOfType(myEffectsList, kParentAtomIsContainer, kEffectNameAtom);
	for (myIndex = 1; myIndex <= myNumEffects; myIndex++) 
	{
		QTAtom				myNameAtom = 0L;
		QTAtom				myTypeAtom = 0L;

		myNameAtom = QTFindChildByIndex(myEffectsList, kParentAtomIsContainer, kEffectNameAtom, myIndex, NULL);
		myTypeAtom = QTFindChildByIndex(myEffectsList, kParentAtomIsContainer, kEffectTypeAtom, myIndex, NULL);
		if ((myNameAtom != 0L) && (myTypeAtom != 0L)) 
		{
			OSType 			myType;
			long			mySize;

			// get the data from the type and name atoms
			QTCopyAtomDataToPtr(myEffectsList, myTypeAtom, false, sizeof(myType), &myType, NULL);
			QTCopyAtomDataToPtr(myEffectsList, myNameAtom, true, 256, dest, &mySize);
			dest[mySize] = '\0';

			if (myType == *effectType)
			{
				return;
			}
		}
	}
	
	//nothing found
	dest[0] = 0;
}

void SavePlaybackHintsToMovie(FSSpecPtr movieFile, bool highQuality, bool singleField)
{
    Movie movie     = NULL;
    Track track     = NULL;
    short resRefNum = 0;
    short resId     = movieInDataForkResID;
    Str255          resName;
    OSErr           err;
	long flags = 0;
	if (highQuality)
		flags |= hintsHighQuality;

	if (singleField)
		flags |= hintsSingleField;


    err = OpenMovieFile( movieFile, &resRefNum, fsWrPerm );
    if (err != noErr) goto bailError;

    err = NewMovieFromFile( &movie, resRefNum,
        &resId, resName, newMovieActive, NULL );
    if (err != noErr) goto bailError;

        /* get the first video track */
    track = GetMovieIndTrackType( movie, 1,
        VisualMediaCharacteristic,
        movieTrackCharacteristic | movieTrackEnabledOnly );
    if ( track == NULL ) goto bailError;

        /* save the play hints with the movie */
    SetTrackLoadSettings( track, 0, 0, 0, flags );
    err = GetMoviesError();
    if (err != noErr) goto bailError;

        /* save our changes to the movie */
    err = UpdateMovieResource( movie,
        resRefNum, movieInDataForkResID, resName );
    if ( movie != NULL )
    {
        DisposeMovie( movie );
    }
    if ( resRefNum != 0 )
    {
        CloseMovieFile( resRefNum );
    }

bailError:
    ;
}

void ReplaceMediaReferences(Movie movie, const char* oldFileName, const char* newFileName)
{
	short mediaDataRefCount;
	Handle	dataRef = NULL;
	OSType 	dataRefType;
	long	dataRefAttributes;
	Str63	string[32];
	char	temp[256];
	char 	fullOldPath[1024];
	Boolean	wasChanged;
	Media media;
	int trackID = 1; //note : these &"'"'(('!( engineers from QT start counting at 1 !!!!!!
	OSErr res;
	long trackCount = GetMovieTrackCount(movie);
	
	while(trackID <= trackCount)
	{
		Track track = GetMovieIndTrack(movie, trackID);
		media = GetTrackMedia(track);
		BeginMediaEdits(media);
		res = GetMediaDataRefCount(media, &mediaDataRefCount);
		
		for (int i=1; i <= mediaDataRefCount; ++i)  //note : these &"'"'(('!( engineers from QT start counting at 1 !!!!!!
		{
			dataRef = NULL;
			
			res = GetMediaDataRef(media, i, &dataRef, &dataRefType, &dataRefAttributes);
			res = GetAliasInfo((AliasHandle)dataRef, 0, string[0]);
			res = GetAliasInfo((AliasHandle)dataRef, 1, string[1]);
			
			if (string[1][0] != 0) // the file is not in the project directory
			{
				//get the full path 
				int directoryLevel = 1;
				fullOldPath[0] = 0;

				do
				{
					directoryLevel++;
					res = GetAliasInfo((AliasHandle)dataRef, directoryLevel, string[directoryLevel]);
				}
				while (string[directoryLevel][0] != 0);
	
				for (int j=directoryLevel-1; j >=0 ; --j)
				{
					CopyPascalStringToC(string[j], temp);
					ZzAppendFileNameToPath(fullOldPath, 1024, fullOldPath, temp);
				}

				//replace by another file
				if (strcmp(fullOldPath, oldFileName) == 0)
				{
					verntl(dataFileRef, 5, "ReplaceMediaReferences : " << oldFileName << " by " << newFileName << " in track " << trackID << " media index = " << i);
					
					FSSpec replace = Path2FSspec(newFileName, false);
					res = UpdateAlias(NULL, &replace, (AliasHandle)dataRef, &wasChanged);
					ThrowOnOSStatus(res);

					res = SetMediaDataRef(media, i, dataRef, dataRefType);
					ThrowOnOSStatus(res);
				}
			}
			DisposeHandle(dataRef);
		}	
		res = EndMediaEdits(media);
		ThrowOnOSStatus(res);
		trackID++;
	}
}

/*bool WriteQTAtomContainerInBuffer(QTAtomContainer* atomContainer, void* buffer, int bufferSize)
{
	QTLockContainer(*atomContainer);
    WriteQTAtomInBuffer(atomContainer, kParentAtomIsContainer, buffer, bufferSize, 0);
	QTUnlockContainer(*atomContainer);

	return true;
}

bool WriteQTAtomInBuffer(QTAtomContainer* atomContainer, QTAtom parentAtom, void* buffer, int bufferSize, int bufferPos)
{
	Ptr tempBuffer;
	long sizeWritten;
    
    QTAtomType atomType = 0;
    while (atomType = QTGetNextChildType(*atomContainer, parentAtom, atomType))
    {
		short atomCount = QTCountChildrenOfType(*atomContainer, parentAtom, atomType);
		for (short i=0; i < atomCount; ++i)
		{
			QTAtom atom = QTFindChildByIndex(*atomContainer, parentAtom, atomType, i+1, NULL);
			OSErr res;
			res = QTGetAtomDataPtr(*atomContainer, atom, &sizeWritten, &tempBuffer);
			if (res == -2102) //the atom is not a leaf atom
			{
				WriteSInt32(-1);
				WriteSInt32(atomType);
				WriteSInt32(parentAtom);
				WriteQTAtom(atomContainer, atom);
				WriteSInt32(0);
			}
			else
			{
				WriteSInt32(atomType);
				WriteSInt32(parentAtom);
				WriteSInt32(sizeWritten);
				WriteRawData(tempBuffer, sizeWritten);
			}	
		}
    }

	return true;
}

bool ArmeArchiveHandler::ReadQTAtomContainer(QTAtomContainer* atomContainer)
{
//	QTLockContainer(*atomContainer);
    ReadQTAtom(atomContainer, kParentAtomIsContainer);
//	QTUnlockContainer(*atomContainer);

	return true;
}

bool ArmeArchiveHandler::ReadQTAtom(QTAtomContainer* atomContainer, QTAtom parentAtom)
{
    SInt32 atomType;
    SInt32 size;
	char* temp;
	QTAtom newAtom;
	OSErr res;		

    while (atomType = ReadSInt32())
    {
		if (atomType == -1) // this is not a leaf atom
		{
			atomType = ReadSInt32();
			ReadSInt32();  //read parent atom (unused)
			res = QTInsertChild(*atomContainer, parentAtom, atomType, 0, 0, 0, 0, &newAtom);
		    ThrowOnOSStatus(res);	
			ReadQTAtom(atomContainer, newAtom);
		}
		else
		{
			ReadSInt32();  //read parent atom (unused)
			size = ReadSInt32();
			temp = new char[size];
			ReadRawData(temp, size);
			res = QTInsertChild(*atomContainer, parentAtom, atomType, 0, 0, size, temp, NULL);
		    ThrowOnOSStatus(res);
			delete temp;
		}
    }
	
	return true;
}*/


